@@ -19,9 +19,16 @@
 
 /* There are systems which store the stdio fp contained fileno into
    a SIGNED CHARACTER, and simple-minded  fileno() will therefore
-   pick FD of over 127 as negative number..  We provide a wrapper
-   for such.. */
-#define FILENO(arg) (unsigned)(fileno(arg))
+   pick an FD of over 127 as negative number..  We provide a wrapper
+   for such in case by case basis.. */
+
+#ifndef FILENO
+# if defined(sun) && !defined(__svr4__)
+#  define FILENO(x) (((unsigned)fileno(x)) & 0xFF)
+# else
+#  define FILENO(x) fileno(x)
+# endif
+#endif
 
 #ifndef	__
 # ifndef __STDC__
