@@ -125,7 +125,7 @@ struct address {
 	struct addr	*a_tokens;		/* RFC822 token list */
 	AddressStamp	a_stamp;		/* what this is */
 	/* XX: this should be changed to a list of tags */
-	short		a_uid;			/* uid for delivery */
+	int		a_uid;			/* uid for delivery */
 	struct address	*a_next;
 	struct notary   *a_dsn;			/* NOTARY data for this addr */
 };
