@@ -66,6 +66,7 @@ struct ctldesc {
 	int		msgfd;		/* message file I/O descriptor */
 	int		ctlfd;		/* control file I/O descriptor */
 	int		ctlid;		/* control file id (inode number) */
+	char		*ctlmap;	/* control file mmap() block */
 	char		*contents;	/* message file data */
 	long		contentsize;	/* message file size */
 	long		*offset;	/* array of indices into contents */
@@ -158,7 +159,7 @@ extern int ermdir __((char *path));
 extern int erename __((char *from, char *to));
 
 /* lockaddr.c: */
-extern int lockaddr __((int fd, long offset, char was, char new));
+extern int lockaddr __((int fd, char *map, long offset, char was, char new));
 
 /* markoff.c: */
 extern int markoff __((char *filecontents, int bytesleft, long offsets[], char *filename));
