@@ -20,11 +20,10 @@
  *  allocated buffer known only within this module. Code external to here
  *  should treat linebuf as a read-only (constant) buffer.
  */
-
-static u_char	*block;		/* malloc()'ed input buffer */
-static u_char	*bend;		/* end of line pointer */
-static u_int	blen;		/* bytes available to consume */
-static u_int	bsize;		/* size of malloc()'ed buffer */
+static u_char	*getline_block;		/* malloc()'ed input buffer */
+static u_char	*getline_bend;		/* end of line pointer */
+static u_int	getline_blen;		/* bytes available to consume */
+static u_int	getline_bsize;		/* size of malloc()'ed buffer */
 
 u_char	*linebuf = NULL;	/* where to start consuming */
 
@@ -37,13 +36,13 @@ void
 initline(blksize)
 	long blksize;
 {
-	if (block == NULL) {
-		bsize = blksize;
-		if (bsize < BUFSIZ)
-			bsize = BUFSIZ;
-		block = (u_char *)emalloc(bsize);
+	if (getline_block == NULL) {
+		getline_bsize = blksize;
+		if (getline_bsize < BUFSIZ)
+			getline_bsize = BUFSIZ;
+		getline_block = (u_char *)emalloc(getline_bsize);
 	}
-	blen = 0;
+	getline_blen = 0;
 }
 
 /*
@@ -57,42 +56,42 @@ getline(fp)
 	register u_char	*cp;
 	register int	n;
 
-	/* assert block != NULL */
-	if (blen <= 0 || bend >= block + bsize) {
-		blen = fread((char *)block, 1, (int)bsize, fp);
-		if (blen == 0)
+	/* assert getline_block != NULL */
+	if (getline_blen <= 0 || getline_bend >= getline_block + getline_bsize) {
+		getline_blen = fread((char *)getline_block, 1, (int)getline_bsize, fp);
+		if (getline_blen == 0)
 			return 0;
-		bend = block;
+		getline_bend = getline_block;
 	}
 	while (1) {
 		/* look for end of line in what remains of the input buffer */
-		for (cp = bend, n = 0; n < blen; ++n, ++cp)
-			if (*cp == '\n') {
-				linebuf = bend;
-				bend = ++cp;
-				blen -= ++n;
-				return n;
-			}
+		for (cp = getline_bend, n = 0; n < getline_blen; ++n, ++cp)
+		  if (*cp == '\n') {
+		    linebuf = getline_bend;
+		    getline_bend = ++cp;
+		    getline_blen -= ++n;
+		    return n;
+		  }
 		/* copy the partial line to the beginning of the input buffer */
-		if (bend > block)
-		  memcpy((char *)block, (char *)bend, (int)blen);
+		if (getline_bend > getline_block)
+		  memcpy((char *)getline_block, (char *)getline_bend, (int)getline_blen);
 		/* get some more bytes which will hopefully contain a newline */
-		n = bsize - blen;
+		n = getline_bsize - getline_blen;
 		if (n <= 0) {		/* grow the buffer */
-			n = bsize;
-			bsize *= 2;
-			block = (u_char *)erealloc((char *)block, bsize);
+			n = getline_bsize;
+			getline_bsize *= 2;
+			getline_block = (u_char *)erealloc((char *)getline_block, getline_bsize);
 		}
-		n = fread((char *)(block+blen), 1, n, fp);
+		n = fread((char *)(getline_block+getline_blen), 1, n, fp);
 		if (n == 0) {
 			/* the file doesn't terminate in a newline */
-			n = blen;
-			blen = 0;
-			linebuf = block;
+			n = getline_blen;
+			getline_blen = 0;
+			linebuf = getline_block;
 			return n;
 		}
-		blen += n;
-		bend = block;
+		getline_blen += n;
+		getline_bend = getline_block;
 	}
 	/* NOTREACHED */
 }
@@ -107,11 +106,11 @@ getline(fp)
 int
 getrest()
 {
-	/* assert block != NULL */
-	if (blen <= 0 || bend >= block + bsize)
+	/* assert getline_block != NULL */
+	if (getline_blen <= 0 || getline_bend >= getline_block + getline_bsize)
 		return 0;
-	linebuf = bend;
-	return blen;
+	linebuf = getline_bend;
+	return getline_blen;
 }
 
 /*
@@ -122,5 +121,5 @@ long
 offset(fp)
 	FILE *fp;
 {
-	return ftell(fp) - blen;
+	return ftell(fp) - getline_blen;
 }
