@@ -21,28 +21,16 @@ rfc822tz(ts, prettyname)
 	int sign, offset;
 
 #ifdef	USE_BSDTIMEZONE
-	struct timezone tz;
-	struct timeval tv;
-	extern char *timezone();
-
-	gettimeofday(&tv, &tz);	/* this can't fail, right?? */
-	offset = tz.tz_minuteswest - (ts->tm_isdst ? 60 : 0);
+	offset = (ts->tm_gmtoff / 60) - (ts->tm_isdst ? 60 : 0);
 	sign = offset >= 0;
 	if (!sign)
-	  offset = -offset;
-#ifdef  NO_TM_ZONE
-	if (prettyname)
-	  cp = timezone(tz.tz_minuteswest,ts->tm_isdst);
-#else   /* !NO_TM_ZONE */
+	  offset = -offset; 	/* if negative, strip sign */
 	if (prettyname)
 	  cp = ts->tm_zone;
-#endif        /* !NO_TM_ZONE */
 	else {
 	  sprintf(zone, "%c%02d%02d",
-		  sign ? '-' : '+', offset / 60, offset % 60);
+		  sign ? '+' : '-', offset / 60, offset % 60);
 	  cp = zone;
-	  if (prettyname)
-	    sprintf(cp+5," (%s)",ts->tm_zone);
 	}
 
 #else	/* !USE_BSDTIMEZONE */
