@@ -22,7 +22,12 @@
 
 /* ISO Latin 1 (8859) */
 
-short	rfc_ctype[256] = {						/* octalcode */
+#ifdef __alpha /* On Alpha the short is slow to access! */
+int
+#else
+short
+#endif
+	rfc_ctype[256] = {						/* octalcode */
 _c,	_c,	_c,	_c,	_c,	_c,	_c,	_c,	/*   0 -   7 */
 _c,	_c|_w,	_l|_c,	_c,	_c,	_r|_c,	_c,	_c,	/*  10 -  17 */
 _c,	_c,	_c,	_c,	_c,	_c,	_c,	_c,	/*  20 -  27 */
