@@ -1019,14 +1019,14 @@ sh_wait(argc, argv)
 	char *argv[];
 {
 	int retcode, pid;
-#if	defined(WNOHANG) && !defined(WLOBYTE)
+#ifdef USE_UNIONWAIT
 	union wait status;
 
 	status.w_termsig = 0;
 	status.w_retcode = 0;
-#else	/* !WNOHANG */
+#else	/* !USE_UNIONWAIT */
 	int status = 0;
-#endif	/* WNOHANG */
+#endif	/* !USE_UNIONWAIT */
 	if (argc > 2 || (argc == 2 && !isdigit(argv[1][0]))) {
 		fprintf(stderr, USAGE_WAIT, argv[0]);
 		return 1;
@@ -1037,7 +1037,7 @@ sh_wait(argc, argv)
 	}
 	if (pid < 0)		/* no more children */
 		return 0;
-#if	defined(WNOHANG) && !defined(WLOBYTE)
+#ifdef	USE_UNIONWAIT
 	if (status.w_termsig) {
 		fprintf(stderr, "%s", strsignal(status.w_termsig));
 		if (status.w_coredump)
@@ -1046,7 +1046,7 @@ sh_wait(argc, argv)
 		retcode = 0200 + status.w_termsig;
 	} else
 		retcode = status.w_retcode;
-#else	/* !WNOHANG */
+#else	/* !USE_UNIONWAIT */
 	if ((status&0177) > 0) {
 		fprintf(stderr, "%s", strsignal(status&0177));
 		if (status&0200)
@@ -1055,7 +1055,7 @@ sh_wait(argc, argv)
 		retcode = 0200 + (status&0177);
 	} else
 		retcode = (status>>8)&0377;
-#endif	/* WNOHANG */
+#endif	/* !USE_UNIONWAIT */
 	return retcode;
 }
 
