@@ -255,9 +255,6 @@ STATIC int
 findfreefd()
 {
         register int fd;
-#ifdef  MAILER
-        struct stat stbuf;
-#endif	/* MAILER */
 
         for (fd = 0; fd <= (sizeof fds / sizeof fds[0]) ; ++fd)
                 if (fds[fd] == 0) {
@@ -1165,7 +1162,8 @@ interpret(code, eocode, entry, caller, retcodep, cdp)
 #define LOOPMAXDEPTH 30
 	struct loopinfo loop[LOOPMAXDEPTH];	/* max # nested loops */
 	struct conscell *variable, *l, *margin;
-	struct osCmd *prevcommand, *pcommand;
+	struct osCmd *prevcommand;
+	struct osCmd *pcommand;
 #define COMMANDMAXDEPTH 30
 	struct osCmd commandStack[COMMANDMAXDEPTH];
 #define VARMAXDEPTH 30
@@ -1708,7 +1706,7 @@ interpret(code, eocode, entry, caller, retcodep, cdp)
 					varmeter->flags = 0;
 					variableIndex--;
 				}
-				if (*command->shcmdp->name == 'r') {
+				if (*command->shcmdp->name == 'r') { /* ``r''eturn */
 #if 0
 					std_printf("idx = %d\n", commandIndex);
 					if (commandIndex >= 0)
@@ -1732,12 +1730,12 @@ nobreak:
 			 * parent command.  Eventually a setlevel() will be
 			 * done anyway to reclaim space... we hope.
 XXX: HERE! Must copy the output to PREVIOUS memory level, then discard
-	   the recursed one! <mea@utu.fi>
+	   the recursed one! <mea@utu.fi> ( command->rval )
 			 */
 			if (command->memlevel && command->rval == NULL
 			    && command->buffer == NULL
 			    && command->iocmd != ioIntoBuffer) {
-				/*setlevel(MEM_SHCMD, command->memlevel);*/
+setlevel(MEM_SHCMD, command->memlevel); /* void routines clean their memory */
 				if (command->pgrp > 0)
 					jc_report(command->pgrp);
 			}
