@@ -62,7 +62,7 @@ char *
 siogets(s)
 	register char *s;
 {
-	register struct siobuf *siop = siofds[fileno(stdin)];
+	register struct siobuf *siop = siofds[FILENO(stdin)];
 	register int ch;
 	char *os = s;
 
@@ -82,7 +82,7 @@ siofgets(s, n, fp)
 	register int n;
 	FILE *fp;
 {
-	register struct siobuf *siop = siofds[fileno(fp)];
+	register struct siobuf *siop = siofds[FILENO(fp)];
 	register int ch;
 	char *os = s;
 
@@ -102,7 +102,7 @@ int
 sioputs(s)
 	register char *s;
 {
-	register struct siobuf *siop = siofds[fileno(stdout)];
+	register struct siobuf *siop = siofds[FILENO(stdout)];
 
 	if (!s)
 		return EOF;
@@ -119,7 +119,7 @@ siofputs(s, fp)
 	register char *s;
 	FILE *fp;
 {
-	register struct siobuf *siop = siofds[fileno(fp)];
+	register struct siobuf *siop = siofds[FILENO(fp)];
 
 	if (!s)
 		return EOF;
@@ -200,7 +200,7 @@ siofread(b, s, n, fp)
 	int s, n;
 	FILE *fp;
 {
-	struct siobuf *siop = siofds[fileno(fp)];
+	struct siobuf *siop = siofds[FILENO(fp)];
 
 	s *= n;
 	n = s;
@@ -217,7 +217,7 @@ siofwrite(b, s, n, fp)
 	register int s, n;
 	FILE *fp;
 {
-	register struct siobuf *siop = siofds[fileno(fp)];
+	register struct siobuf *siop = siofds[FILENO(fp)];
 
 	if (!b)
 		return EOF;
