@@ -155,7 +155,7 @@ ShInitIFS(ifs)
 			++count;
 		}
 	/* Using strsave() has hidden dangers due to  stickymem  value! */
-	saveifs = (u_char *)emalloc(strlen(ifs)+1);
+	saveifs = (u_char *)emalloc(strlen((char*)ifs)+1);
 	strcpy((char *)saveifs, (char *)ifs);
 }
 
