@@ -50,6 +50,7 @@ struct db_info {
 	int		(*add)();		/* add key/value pair */
 	int		(*remove)();		/* remove key */
 	void		(*print)();		/* print database */
+	void		(*count)();		/* count database */
 	void		(*owner)();		/* print database owner */
 	int		(*modcheckp)();		/* should we reopen database? */
 	postprocs	postproc;		/* post-lookup applicator */
@@ -67,45 +68,45 @@ struct db_kind {
 	struct db_info	config;		/* default configuration information */
 } db_kinds[] = {
 { "incore",	{ NULL, NULL, 0, 0, 0, NULL, search_core, close_core,
-		  add_core, remove_core, print_core, owner_core, NULL, Nul } },
+		  add_core, remove_core, print_core, count_core, owner_core, NULL, Nul } },
 { "header",	{ NULL, NULL, 0, 0, 0, NULL, search_header, close_header,
-		  add_header, remove_header, print_header, owner_header, NULL, Nul } },
+		  add_header, remove_header, print_header, count_header, owner_header, NULL, Nul } },
 { "unordered",	{ NULL, NULL, 0, 10, 0, NULL, search_seq, close_seq,
-		  add_seq, NULL, print_seq, owner_seq, modp_seq, Nul } },
+		  add_seq, NULL, print_seq, count_seq, owner_seq, modp_seq, Nul } },
 #ifndef	USE_MMAP
 { "ordered",	{ NULL, NULL, 0, 10, 0, NULL, search_bin, close_seq,
-		  NULL, NULL, print_seq, owner_seq, modp_seq, Nul }	},
+		  NULL, NULL, print_seq, count_seq, owner_seq, modp_seq, Nul }	},
 #else /* USE_MMAP */ /* When using MMAP(), no cache is needed for ordered.. */
 { "ordered",	{ NULL, NULL, 0, 0, 0, NULL, search_bin, close_seq,
-		  NULL, NULL, print_seq, owner_seq, modp_seq, Nul }	},
+		  NULL, NULL, print_seq, count_seq, owner_seq, modp_seq, Nul }	},
 #endif
 
 #ifdef	USE_HOSTS
 { "hostsfile",	{ HOSTS_FILE, NULL, 0, 10, 0, NULL, search_hosts, NULL,
-		  NULL, NULL, print_hosts, NULL, NULL, Nul } },
+		  NULL, NULL, print_hosts, NULL, NULL, NULL, Nul } },
 #endif	/* USE_HOSTS */
 #ifdef	USE_RESOLV
 { "bind",	{ RESOLV_CONF, NULL, 0, 10, 0, NULL,
-		  search_res, NULL, NULL, NULL, NULL, NULL, NULL, Nul }	},
+		  search_res, NULL, NULL, NULL, NULL, NULL, NULL, NULL, Nul }	},
 #endif	/* USE_RESOLV */
 #ifdef	USE_NDBM
 { "ndbm",	{ NULL, NULL, 0, 10, 0, NULL, search_ndbm, close_ndbm,
-		  add_ndbm, remove_ndbm, print_ndbm, owner_ndbm,
+		  add_ndbm, remove_ndbm, print_ndbm, count_ndbm, owner_ndbm,
 		  modp_ndbm, Nul } },
 #endif	/* USE_NDBM */
 #ifdef	USE_GDBM
 { "gdbm",	{ NULL, NULL, 0, 10, 0, NULL, search_gdbm, close_gdbm,
-		  add_gdbm, remove_gdbm, print_gdbm, owner_gdbm,
+		  add_gdbm, remove_gdbm, print_gdbm, count_gdbm, owner_gdbm,
 		  modp_gdbm, Nul } },
 #endif	/* USE_GDBM */
 #ifdef	USE_DBM
 { "dbm",	{ NULL, NULL, 0, 10, 0, NULL, search_dbm, close_dbm,
-		  add_dbm, remove_dbm, print_dbm, owner_dbm,
+		  add_dbm, remove_dbm, print_dbm, count_dbm, owner_dbm,
 		  NULL, Nul } },
 #endif	/* USE_DBM */
 #ifdef	USE_YP
 { "yp",		{ NULL, NULL, 0, 10, 0, NULL, search_yp, NULL,
-		  NULL, NULL, NULL, NULL, NULL, Nul }	},
+		  NULL, NULL, print_yp, NULL, owner_yp, NULL, Nul }	},
 #endif	/* USE_YP */
 { 0 }	/* proto_config is initialized from this entry */
 };
@@ -301,6 +302,8 @@ run_relation(argc, argv)
 		proto_config.remove = dbkp->config.remove;
 	if (proto_config.print == NULL)
 		proto_config.print = dbkp->config.print;
+	if (proto_config.count == NULL)
+		proto_config.count = dbkp->config.count;
 	if (proto_config.owner == NULL)
 		proto_config.owner = dbkp->config.owner;
 	if (proto_config.modcheckp == NULL)
@@ -348,6 +351,7 @@ icdbspltree(name)
  *		db remove name key
  *		db flush name
  *		db print name
+ *		db count name
  *		db index
  *		db owner name
  */
@@ -456,12 +460,13 @@ run_db(argc, argv)
 		case 'f':
 		case 'n':
 		case 'o':
+		case 'c':
 		case 'p':	errflag = (argc != 3); break;
 		default:	errflag = 1; break;
 		}
 	if (errflag) {
 		fprintf(stderr,
-"Usage: %s { add|remove|flush|owner|print|toc } [ database [ key [ value ] ] ]\n",
+"Usage: %s { add|remove|flush|owner|print|count|toc } [ database [ key [ value ] ] ]\n",
 			argv[0]);
 		return 1;
 	}
@@ -564,6 +569,14 @@ run_db(argc, argv)
 		}
 		(*dbip->print)(&si, stdout);
 		break;
+	case 'c':	/* count db */
+		if (dbip->count == NULL) {
+			fprintf(stderr, "%s: %s: no counting capability!\n",
+					argv[0], argv[2]);
+			return 1;
+		}
+		(*dbip->count)(&si, stdout);
+		break;
 	default:
 		fprintf(stderr, "%s: unknown command '%s'\n", argv[0], argv[1]);
 		return 5;
