@@ -72,6 +72,8 @@ struct headerinfo envelope_hdrs[] = {
 { "verbose",		AnyWord,	nilUserType,	eVerbose	},
 { "via",		AnyWord,	nilUserType,	eVia		},
 { "with",		AnyWord,	nilUserType,	eWith		},
+{ "env-eof",	nilHeaderSemantics,	nilUserType,	eEnvEnd		},
+{ "env-end",	nilHeaderSemantics,	nilUserType,	eEnvEnd		},
 };
 
 struct semtrans {
@@ -431,7 +433,7 @@ remove_header(sip)
 
 static FILE *pcfp;
 
-int
+static int
 hdprintdata(spl)
 	struct spblk *spl;
 {
@@ -447,6 +449,7 @@ hdprintdata(spl)
 	return 0;
 }
 
+
 void
 print_header(sip, outfp)
 	struct search_info *sip;
@@ -461,6 +464,32 @@ print_header(sip, outfp)
 	fflush(outfp);
 }
 
+/* Count the database */
+
+static int   pc_cnt;
+
+static int
+hdcountdata(spl)
+	struct spblk *spl;
+{
+	++pc_cnt;
+	return 0;
+}
+
+void
+count_header(sip, outfp)
+	struct search_info *sip;
+	FILE *outfp;
+{
+	struct sptree *db;
+	pc_cnt = 0;
+
+	if ((db = open_header(sip)) != NULL)
+	  sp_scan(hdcountdata, (struct spblk *)NULL, db);
+	fprintf(outfp,"%d\n",pc_cnt);
+	fflush(outfp);
+}
+
 void
 owner_header(sip, outfp)
 	struct search_info *sip;
