@@ -68,8 +68,12 @@ reopen:
 		fm->mtime = fst.st_mtime;
 		fm->lines = 0;
 		fm->offsets = NULL;
-		fm->membuf = (void*)mmap(NULL, fst.st_size,
-					 PROT_READ, MAP_SHARED, FILENO(fp), 0);
+		if (fm->size)
+		  fm->membuf = (void*)mmap(NULL, fst.st_size,
+					   PROT_READ, MAP_SHARED,
+					   FILENO(fp), 0);
+		else
+		  fm->membuf = NULL;
 		if ((long)fm->membuf == -1) {
 			fprintf(stderr,
 				"search_bin: cannot mmap() (r/o) %d bytes of file \"%s\" into memory, errno=%d",
@@ -112,7 +116,7 @@ reopen:
 	fm->pos = 0;	/* We have it mmap()ed incore, collect line
 			   start offsets into an array for latter use
 			   on searches..				*/
-	if (fm->lines == 0 && fm->offsets == NULL) {
+	if (fm->size > 0 && fm->lines == 0 && fm->offsets == NULL) {
 	  int linecnt = 0;
 	  char *buf = fm->membuf;
 	  char *eof = fm->membuf + fm->size;
