@@ -68,16 +68,20 @@ reopen:
 		fm->mtime = fst.st_mtime;
 		fm->lines = 0;
 		fm->offsets = NULL;
-		fm->membuf = (void*)mmap(NULL, fst.st_size,
-					 PROT_READ, MAP_SHARED, FILENO(fp), 0);
+		if (fm->size)
+		  fm->membuf = (void*)mmap(NULL, fst.st_size,
+					   PROT_READ, MAP_SHARED, FILENO(fp),
+					   0);
+		else
+		  fm->membuf = NULL;
 		if ((long)fm->membuf == -1) {
-			fprintf(stderr,
-				"search_seq: cannot mmap() (r/o) %d bytes of file \"%s\" into memory, errno=%d",
-				fst.st_size, sip->file, errno);
-			free(fm);
-			++deferit;
-			v_set(DEFER, DEFER_IO_ERROR);
-			return NULL;
+		  fprintf(stderr,
+			  "search_seq: cannot mmap() (r/o) %d bytes of file \"%s\" into memory, errno=%d",
+			  fst.st_size, sip->file, errno);
+		  free(fm);
+		  ++deferit;
+		  v_set(DEFER, DEFER_IO_ERROR);
+		  return NULL;
 		}
 #else
 		fm->size   = 0;
@@ -138,7 +142,7 @@ reopen:
 		fclose(fp);
 #ifdef	USE_MMAP
 		if (fm->membuf != NULL)
-			munmap(fm->membuf,fm->size);
+		  munmap(fm->membuf,fm->size);
 #endif
 		if (fm->offsets != NULL)
 		  free(fm->offsets);
@@ -176,7 +180,7 @@ close_seq(sip)
 	fclose(fm->fp);
 #ifdef	USE_MMAP
 	if (fm->membuf != NULL)
-		munmap(fm->membuf,fm->size);
+	  munmap(fm->membuf,fm->size);
 #endif
 	if (fm->offsets != NULL)
 	  free(fm->offsets);
@@ -226,8 +230,12 @@ open_seq(sip, mode)
 		fm->mtime = fst.st_mtime;
 		fm->lines = 0;
 		fm->offsets = NULL;
-		fm->membuf = (void*)mmap(NULL, fst.st_size,
-					 PROT_READ, MAP_SHARED, FILENO(fm->fp), 0);
+		if (fm->size)
+		  fm->membuf = (void*)mmap(NULL, fst.st_size,
+					   PROT_READ, MAP_SHARED,
+					   FILENO(fm->fp), 0);
+		else
+		  fm->membuf = NULL;
 #else
 		fm->size   = 0;
 		fm->mtime  = 0;
@@ -287,6 +295,56 @@ print_seq(sip, outfp)
 	fflush(outfp);
 }
 
+/*
+ * Count the database.  This is equivalent to listing the file and so it
+ * can be used by the other text-reading database types, e.g. search_bin().
+ */
+
+void
+count_seq(sip, outfp)
+	struct search_info *sip;
+	FILE *outfp;
+{
+	FILE *fp;
+	int n;
+	char buf[BUFSIZ];
+	int cnt = 0;
+
+	if ((fp = open_seq(sip, "r")) != NULL) {
+#ifdef USE_MMAP_not
+	  struct file_map *fm;
+	  spkey_t symid = symbol((u_char *)sip->file);
+	  struct spblk *spl = sp_lookup(symid, spt_files);
+	  int cnt;
+
+	  if (spl == NULL) { /* XX: HOW ?? We have the file open! */
+	    abort();
+	  }
+	  fm = (struct file_map *)spl->data;
+
+	  if (fm->size > 0 && fm->lines == 0) /* not yet counted ? */ {
+	    char *p   =     fm->membuf;
+	    char *eop = p + fm->size;
+	    for (;p < eop; ++p)
+	      if (*p == '\n')
+		++cnt;
+	    fm->lines = cnt;
+	  }
+	  cnt = fm->lines;
+#else
+	  fseek(fp, 0L, 0);
+	  while ((n = fread(buf, 1, sizeof buf, fp)) > 0) {
+	    while (n >= 0) {
+	      if (buf[n] == '\n') ++cnt;
+	      --n;
+	    }
+	  }
+#endif
+	}
+	fprintf(outfp,"%d\n",cnt);
+	fflush(outfp);
+}
+
 void
 owner_seq(sip, outfp)
 	struct search_info *sip;
@@ -346,9 +404,12 @@ struct file_map *fm;
 long newsize;
 {
 	if (fm->membuf != NULL)
-		munmap(fm->membuf, fm->size);
-	fm->membuf = (void*)mmap(NULL, newsize,
-				 PROT_READ, MAP_SHARED, FILENO(fm->fp), 0);
+	  munmap(fm->membuf, fm->size);
+	if (newsize)
+	  fm->membuf = (void*)mmap(NULL, newsize,
+				   PROT_READ, MAP_SHARED, FILENO(fm->fp), 0);
+	else
+	  fm->membuf = NULL;
 	fm->size = newsize;
 	if (fm->offsets != NULL)
 	  free(fm->offsets);
@@ -368,6 +429,7 @@ struct file_map *fm;
 	char *b   = buf;
 	int i;
 
+	if (fm->size == 0) return NULL; /* no buffer mapped.. */
 	if (s >= eof) return NULL; /* EOF.. */
 
 	for (i = 0; *s != '\n' && s < eof && i < bufsize; ++i)
@@ -426,10 +488,10 @@ reopen:
 		fm->offsets = NULL;
 		fm->membuf = NULL;
 		if (spl == NULL)
-			spl = sp_install(symid, (u_char *)fm,
-					 O_RDONLY, spt_files);
+		  spl = sp_install(symid, (u_char *)fm,
+				   O_RDONLY, spt_files);
 		else
-			spl->data = (u_char *)fm;
+		  spl->data = (u_char *)fm;
 	}
 	fp = fm->fp;
 
