@@ -1942,7 +1942,7 @@ prctladdr(info, fp, cfflag, comment)
 			} else
 				x = l;
 			if (x != NULL) {
-				if (*x->string == '\0')
+				if (x->string == NULL || *x->string == '\0')
 					putc('-', fp);
 				else
 					fprintf(fp, "%s", x->string);
