@@ -274,11 +274,12 @@ main(argc, argv)
 		      break;
 		    /* parent */
 		  }
-		  if (router_id == nrouters + 1)
-		    router_id = 1;
 		  printf("%s: number %d started as pid %d\n",
 			 progname, router_id, getpid());
+		  router_id = getpid();
 		}
+		else
+		  router_id = getpid();
 #ifdef	LOG_PID
 #ifdef	LOG_MAIL
 		openlog("router", LOG_PID, LOG_MAIL);
