@@ -21,7 +21,7 @@ doagenda()
 	int didsomething;
 	struct thread *ncuritem, *nncuritem;
 
-	time(&now);
+	mytime(&now);
 
 	ncuritem = thread_head;
 	didsomething = 0;
@@ -47,7 +47,7 @@ doagenda()
 	  queryipccheck();
 	} while (didsomething && ncuritem);
 
-	time(&now);
+	mytime(&now);
 
 	/* if (verbose)
 	   printf("alarmed %d\n", now);  */
