@@ -415,7 +415,7 @@ checkrouter()
 	  /* if the .router.pid file is younger than any core file,
 	     then the router dumped core... so let'em know about it. */
 	  sprintf(path, "%s/%s/core",postoffice,ROUTERDIR);
-	  if (fstat(fileno(fp), &pidbuf) < 0) {
+	  if (fstat(FILENO(fp), &pidbuf) < 0) {
 	    fprintf(stderr, "\n%s: ", progname);
 	    perror("fstat: ");
 	  } else if (stat(path, &corebuf) == 0 && pidbuf.st_mtime < corebuf.st_mtime)
@@ -874,9 +874,13 @@ report(fp)
 	if (!parse(fp))
 	  return;
 	if (schedq) {
-	  int c;
-	  while ((c = getc(fp)) != EOF) {
-	    putc(c,stdout);
+	  char linebuf[4000];
+	  while (!feof(fp) && !ferror(fp)) {
+	    if (fgets(linebuf,sizeof(linebuf),fp) == NULL)
+	      break;
+	    if (schedq > 1 && linebuf[0] == '\t' && linebuf[1] != '\t')
+	      continue;
+	    fputs(linebuf,stdout);
 	  }
 	  return;
 	}
