@@ -159,6 +159,35 @@ msgerror(vp, offset, message)
 	fclose(fp);
 }
 
+/* Pick recipient address from the input line.
+   EXTREMELY Simple minded parsing.. */
+static void pick_env_addr(buf,mfp)
+char *buf;
+FILE *mfp;
+{
+	char *s = buf;
+
+	while (*s != 0 && *s != ' ' && *s != '\t' && *s != ':') ++s;
+	if (*s != ':') return; /* BAD INPUT! */
+	buf = ++s; /* We have skipped the initial header.. */
+
+	s = strchr(buf,'\n');
+	if (s) *s = 0;
+	s = strchr(buf,'<');
+	if (s != NULL) {
+	  /*  Cc:  The Postoffice managers <postoffice> */
+	  buf = ++s;
+	  s = strrchr(buf,'>');
+	  if (s) *s = 0;
+	  else return; /* No trailing '>' ? BAD BAD! */
+	  fprintf(mfp,"to %s\n",buf);
+	} else {
+	  /*  Cc: some-address  */
+	  fprintf(mfp,"to %s\n",buf);
+	}
+}
+
+
 /* called to process errors at sensible intervals */
 
 void
@@ -224,15 +253,32 @@ reporterrs(cfpi)
 	    if (*eaddr == '|' || *eaddr == '/')
 	      eaddr = "postmaster"; /* Paranoid, eh ? */
 	    fprintf(errfp, "channel error\n");
-	    fprintf(errfp, "To: %s\n", eaddr);
+
+	    fprintf(errfp, "to %s\n", eaddr);
 	    sprintf(path, "%s/%s/%s", mailshare, FORMSDIR,
 		    deliveryform ? deliveryform : "delivery");
 	    if ((fp = fopen(path, "r")) != NULL) {
 	      int inhdr = 1;
 	      buf[sizeof(buf)-1] = 0;
+	      while (fgets(buf,sizeof(buf)-1,fp) != NULL) {
+		if (strncmp(buf,"HDR",3)==0)
+		  continue;
+		else if (strncmp(buf,"ADR",3)==0)
+		  pick_env_addr(buf+4,errfp);
+		else if (strncmp(buf,"SUB",3)==0)
+		  continue;
+		else
+		  break;
+	      }
+	      fputs("env-end\n",errfp); /* Envelope ends */
+	      fseek(fp,0,0);		/* Rewind! Start building the report */
+
+	      fprintf(errfp, "To: %s\n", eaddr);
 	      while (fgets(buf,sizeof(buf)-1,fp) != NULL) {
 		if (strncmp(buf,"HDR",3)==0) {
 		  fputs(buf+4,errfp);
+		} else if (strncmp(buf,"ADR",3)==0) {
+		  fputs(buf+4,errfp);
 		} else if (strncmp(buf,"SUB",3)==0) {
 		  fputs(buf+4,errfp);
 		} else {
@@ -325,7 +371,7 @@ reporterrs(cfpi)
 	    putc('\n', errfp);
 	  } else
 	    fprintf(errfp, "%s\n", cp);
-	  lockaddr(cfp->fd, *lp + 1, _CFTAG_NORMAL, _CFTAG_OK);
+	  lockaddr(cfp->fd, NULL, *lp + 1, _CFTAG_NORMAL, _CFTAG_OK);
 #ifdef	LOG_INFO
 	  syslog(LOG_INFO, "%s: <%s>: %s",
 		 cfp->mid, cfp->contents + byteidx + 2, cp);
@@ -343,7 +389,8 @@ reporterrs(cfpi)
 	}
 	if (envid != NULL)
 	  fprintf(errfp, "Original-Envelope-Id: %s\n",envid);
-	fprintf(errfp, "Arrival-Date: %s\n", rfc822date(&cfpi->ctime));
+	/* rfc822date() returns a string with trailing newline! */
+	fprintf(errfp, "Arrival-Date: %s", rfc822date(&cfpi->ctime));
 	fprintf(errfp, "\n");
 
 	/* Now scan 'em all again for IETF-NOTARY */
