@@ -90,6 +90,7 @@ extern time_t now;
 extern FILE *statuslog;
 extern void resync_file __((char *filename));
 extern int thread_count_recipients __((void));
+extern time_t mytime __((time_t *));
 
 /* threads.c */
 extern struct thread *thread_head, *thread_tail;
@@ -101,7 +102,7 @@ extern struct web *web_findcreate __((int flag, u_char *s));
 extern void        unweb __((int flag, struct web *wp));
 extern int   thread_start __((struct thread *thr));
 extern int   idle_cleanup __((void));
-extern void  pick_next_vertex __((struct procinfo *proc, int ok));
+extern void  pick_next_vertex __((struct procinfo *proc, int ok, int justfree));
 extern void  thread_report __((FILE *));
 extern int   idleprocs;
 extern void  web_disentangle __((struct vertex *vp, int ok));
@@ -112,6 +113,7 @@ extern void  thread_reschedule __((struct thread *, time_t, int index));
 extern struct procinfo *cpids;
 extern int  numkids;
 extern void feed_child      __((struct procinfo *cpidp));
+extern int flush_child      __((struct procinfo *cpidp));
 extern void idle_child      __((struct procinfo *cpidp));
 extern int  start_child     __((struct vertex *vhead, struct web *channel, struct web *host));
 extern SIGNAL_TYPE sig_chld __((int sig));
