@@ -18,10 +18,10 @@
 /* define NO_VERBOSE_MAILQ, if you don't want to output potentially HUGE
    mailq report -- necessary at some sites, while definitely undesirable
    at others..  We need to change this mechanism.. */
+extern int mailq_Q_mode; /* Argument 'Q' means: output only 'mailq -Q' -data.. */
 /* #define NO_VERBOSE_MAILQ */
 
 extern int errno;
-extern time_t time();
 
 static int qpctlfile();
 static int qpchannel();
@@ -79,18 +79,21 @@ qprint(fd)
 	qid  = 0;
 
 	fprintf(qpfp, "version zmailer 1.0\nVertices:\n");
-	time(&qpnow);
+	mytime(&qpnow);
 #ifndef NO_VERBOSE_MAILQ
-	sp_scan(qpctlfile, (struct spblk *)NULL, spt_mesh[L_CTLFILE]); 
+	if (!mailq_Q_mode)
+	  sp_scan(qpctlfile, (struct spblk *)NULL, spt_mesh[L_CTLFILE]); 
 #endif
 	if (qpch != '\0') {
 		fprintf(qpfp, "Channels:\n");
 #ifndef NO_VERBOSE_MAILQ
-		sp_scan(qpchannel, (struct spblk *)NULL, spt_mesh[L_CHANNEL]);
+		if (!mailq_Q_mode)
+		  sp_scan(qpchannel, (struct spblk *)NULL, spt_mesh[L_CHANNEL]);
 #endif
 		fprintf(qpfp, "Hosts:\n");
 #ifndef NO_VERBOSE_MAILQ
-		sp_scan(qphost, (struct spblk *)NULL, spt_mesh[L_HOST]);
+		if (!mailq_Q_mode)
+		  sp_scan(qphost, (struct spblk *)NULL, spt_mesh[L_HOST]);
 #endif
 	}
 	fprintf(qpfp, "End:\n");
@@ -101,7 +104,7 @@ qprint(fd)
 
 	/* XX: I suppose we don't really need to do this if we use TCP. */
 	if (rendezvous != NULL) {
-	  time_t now = time((time_t *)0) - 1;
+	  time_t now = mytime(NULL) - 1;
 #ifdef	USE_TIMEVAL
 	  tvp[0].tv_sec = tvp[1].tv_sec = now;
 	  tvp[0].tv_usec = tvp[1].tv_usec = 0;
@@ -162,14 +165,18 @@ static int qpctlfile(spl)
 		  fprintf(qpfp,"touched");
 	    } else {
 	      fprintf(qpfp, "(activation pending, thread");
-	      if (vp->thread->proc) {
+	      /* A vertex is always on some thread.. */
+	      if (vp->thread && vp->thread->proc) {
 		fprintf(qpfp," pid=%d ", vp->thread->proc->pid);
 		if (vp->thread->proc->thread == vp->thread)
 		  fprintf(qpfp,"expected");
 		else
 		  fprintf(qpfp,"bygone");
 	      } else {
-		fprintf(qpfp," inactive");
+		if (vp->thread == NULL)
+		  fprintf(qpfp," NO_THREAD!");
+		else
+		  fprintf(qpfp," inactive");
 	      }
 	    }
 	    ++i;
