@@ -24,6 +24,9 @@
 #include <string.h>
 #include NDIR_H
 #include "zmsignal.h"
+#ifdef USE_MMAP
+#include <sys/mman.h>
+#endif
 
 #include "prototypes.h"
 
@@ -68,8 +71,11 @@ int	querysocket = -1;	/* fd of TCP socket to listen for queries */
 int	D_alloc = 0;
 int	hungry_childs = 0;
 int	global_wrkcnt = 0;
+int	mailq_Q_mode = 0;
 int	syncstart = 0;		/* while set, the thread subsystem shall start
 				   no childs! */
+extern int forkrate_limit;	/* How many forks per second ? */
+
 static int vtxprep_skip      = 0;
 static int vtxprep_skip_any  = 0;
 static int vtxprep_skip_lock = 0;
@@ -109,7 +115,15 @@ extern int estat __((char *filename, struct stat *stbuf));
 extern int eunlink __((char *filename));
 extern int optind;
 
-extern time_t time();
+static int timeserver_pid = 0;
+extern time_t mytime __((time_t *));
+static void init_timeserver __((void));
+
+#ifdef USE_TIMEVAL
+extern int gettimeofday();
+#else
+extern time_t time __((time_t *));
+#endif
 extern u_char *rfc822date __((time_t *));
 extern void detach __((void));
 extern void die __((int rc, char *reason));
@@ -162,7 +176,7 @@ struct spblk   *spl;
 	  for (vp = cfp->head; vp != NULL; vp = nvp) {
 	    nvp = vp->next[L_CTLFILE];
 	    vp->ngroup = 0;
-	    unvertex(vp,1,1); /* Don't unlink()! Just free()! */
+	    unvertex(vp,-1,1); /* Don't unlink()! Just free()! */
 	  }
 	} else {
 	  free_cfp_memory(cfp);
@@ -211,7 +225,7 @@ main(argc, argv)
 
 	/* setlinebuf(stderr);  -- no need for this ? */
 
-	time(&sched_starttime);
+	mytime(&sched_starttime);
 
 	memset(&dqb,0,sizeof(dqb));
 	dirscan_mesh = sp_init();
@@ -235,7 +249,7 @@ main(argc, argv)
 	dlyverbose = 0;
 	syncstart = 0;
 	verbose = errflg = version = 0;
-	while ((c = getopt(argc, argv, "divE:f:Fl:L:N:P:Q:SVW")) != EOF)
+	while ((c = getopt(argc, argv, "divE:f:Fl:L:N:P:q:QR:SVW")) != EOF)
 		switch (c) {
 		case 'f':	/* override default config file */
 			config = optarg;
@@ -265,9 +279,17 @@ main(argc, argv)
 		case 'P':	/* override default postoffice */
 			postoffice = optarg;
 			break;
-		case 'Q':	/* override default mail queue rendezvous */
+		case 'Q':
+			mailq_Q_mode = 1;
+			break;
+		case 'q':	/* override default mail queue rendezvous */
 			rendezvous = optarg;
 			break;
+		case 'R':	/* How many new childs per second to be spawned ? */
+			forkrate_limit = atoi(optarg);
+			if (forkrate_limit < 1)
+			  forkrate_limit = 1;
+			break;
 		case 'v':	/* be verbose and synchronous */
 			++verbose;
 			daemonflg = 0;
@@ -377,7 +399,7 @@ main(argc, argv)
 	    /* NOTREACHED */
 	  }
 	  detach();		/* leave worldy matters behind */
-	  time(&now);
+	  mytime(&now);
 	  printf("%s: scheduler daemon (%s)\n\tstarted at %s\n",
 		 progname, Version, (char *)rfc822date(&now));
 	}
@@ -422,8 +444,11 @@ main(argc, argv)
 	SIGNAL_HANDLE(SIGALRM, sig_alarm);	/* process agenda */
 	SIGNAL_HANDLE(SIGUSR2, sig_iot);	/* dump queue info */
 
-	queryipcinit();
+#ifdef USE_MMAP /* Either this, or shared segment, or thread.. */
+	init_timeserver();
+#endif
 
+	queryipcinit();
 
 	dirqueuescan(".", dq);
 
@@ -444,7 +469,7 @@ main(argc, argv)
 	  int startcount;
 	  dirqueuescan(".", dq);
 	  startcount = dq->wrkcount;
-	  while (dq->wrkcount > 0) {
+	  while (dq->wrkcount > 0 && !mustexit) {
 	    syncweb(dq, tp);
 	    queryipccheck();
 	  }
@@ -457,7 +482,7 @@ main(argc, argv)
 	do {
 	  time_t timeout;
 
-	  time(&now);
+	  mytime(&now);
 
 	  canexit = 0;
 
@@ -511,8 +536,9 @@ main(argc, argv)
 	      tp = rereadconfig(tp, config);
 	      rereadcf = 0;
 	    }
+	    mytime(&now);
 	  } while (mux(timeout) == 0 && !gotalarm &&
-		   dq->wrkcount == 0 && !mustexit);
+		   dq->wrkcount == 0 && !mustexit && timeout > now);
 	} while (!mustexit);
 
 	/* Doing nicely we would kill the childs here, but we are not
@@ -747,7 +773,6 @@ static int syncweb(dq, cehead)
 	    if (schedule(fd, file, cehead, ino) != NULL) {
 	      /* Success, increment counters */
 	      ++wrkcnt;
-	      ++global_wrkcnt;
 	    }
 	  }
 	}
@@ -827,13 +852,15 @@ static struct ctlfile *schedule(fd, file, cehead, ino)
 	    fseek(cfp->vfp, 0, 2);
 	}
 	/* If no verbose trace, don't keep contents in core!
-	   this means HUGE memory save at some machines! */
+	   this means HUGE memory save at busy machines! */
 	if (cfp->vfp == NULL)
 	  if (cfp->contents != NULL) {
 	    free(cfp->contents);
 	    cfp->contents = NULL;
 	  }
 
+	sp_install(cfp->id, (u_char *)cfp, 0, spt_mesh[L_CTLFILE]);
+	++global_wrkcnt;
 	return cfp;
 }
 
@@ -1021,7 +1048,6 @@ static struct ctlfile *vtxprep(cfp)
 	if (cfp == NULL)
 	  return NULL;
 
-	sp_install(cfp->id, (u_char *)cfp, 0, spt_mesh[L_CTLFILE]);
 	channel = host = NULL;
 
 	/* copy offsets into local array */
@@ -1652,10 +1678,103 @@ timestring()
 	static char timebuf[40];
 	struct tm *tp;
 
-	time(&now);
+	mytime(&now);
 	tp = localtime(&now);
 	sprintf(timebuf,"%d%02d%02d%02d%02d%02d",
 		tp->tm_year+1900, tp->tm_mon+1, tp->tm_mday,
 		tp->tm_hour, tp->tm_min, tp->tm_sec);
 	return timebuf;
 }
+
+#ifdef USE_MMAP
+struct timeserver {
+	int	pid;
+#ifdef USE_TIMEVAL
+	struct timeval tv;
+#else
+	time_t	time_sec;
+#endif
+};
+struct timeserver *timeserver_segment = NULL;
+void init_timeserver()
+{
+	int ppid;
+
+#ifndef MAP_ANONYMOUS /* must have a file ? (SunOS 4.1) */
+	char blk[1024];
+	int i;
+	FILE *fp = tmpfile();
+	if (!fp) return; /* Brr! */ 
+
+	memset(blk,0,sizeof(blk));
+	for (i=0; i < 64; ++i)
+	  write(fileno(fp),blk,sizeof(blk));
+
+	timeserver_segment = mmap(NULL, 65536, PROT_READ|PROT_WRITE,
+				  MAP_SHARED, fileno(fp), 0);
+	fclose(fp);
+	
+#else
+	/* This MAP_ANONYMOUS does work at DEC OSF/1 ... */
+	timeserver_segment = mmap(NULL, 65536, PROT_READ|PROT_WRITE,
+				  MAP_VARIABLE|MAP_ANONYMOUS|MAP_SHARED,
+				  -1, 0);
+#endif
+	if (-1L == (long)timeserver_segment
+	    || timeserver_segment == NULL) {
+	  perror("mmap() of timeserver segment gave");
+	  timeserver_segment = NULL;
+	  return; /* Brr.. */
+	}
+
+	ppid = fork();
+	if (ppid > 0) {
+	  timeserver_pid = ppid;
+	  timeserver_segment->pid = ppid;
+	  return;
+	}
+	if (ppid < 0) return; /* Error ?? brr.. */
+
+	ppid = getppid(); /* who is our parent ? */
+
+	for(;;) {
+#ifdef USE_SELECT
+	  struct timeval tv;
+	  int rc;
+
+	  tv.tv_sec = 1;
+	  tv.tv_usec = 0;
+
+	  gettimeofday(&timeserver_segment->tv,NULL);
+
+	  rc = select(0,NULL,NULL,NULL,&tv);
+#else
+	  time(&timeserver_segment->time_sec);
+	  sleep(1);
+#endif
+	  /* Is the parent still alive ?? */
+	  if (kill(ppid, 0) != 0)
+	    break; /* No ?? Out! */
+	}
+	_exit(0);
+}
+#endif
+
+time_t mytime(timep)
+time_t *timep;
+{
+#ifdef USE_MMAP
+	if (timeserver_pid) {
+	  time_t t;
+#ifdef USE_TIMEVAL
+	  t = timeserver_segment->tv.tv_sec;
+#else
+	  t = timeserver_segment->time_sec;
+#endif
+	  if (timep != NULL)
+	    *timep = t;
+	  return t;
+	}
+#endif
+	return time(timep); /* The classical old version.. */
+}
