@@ -292,7 +292,7 @@ int ok;
 	      if (thr1->proc == NULL ||
 		  thr1->proc->thread != thr1) {
 		/* Ok, this thread is not busy, choose it! */
-		time(&now);
+		mytime(&now);
 		thread_start(thr1);
 		break;
 	      }
@@ -365,7 +365,7 @@ void (*ce_fillin) __((struct threadgroup*, struct config_entry *));
 	struct web *wc = vp->orig[L_CHANNEL];
 	struct web *wh = vp->orig[L_HOST];
 
-	time(&now);
+	mytime(&now);
 
 if (verbose) printf("thread_linkin([%s/%s],%s/%d/%s,%d)\n",wc->name,wh->name,
 		    cep->channel,cep->flags & CFG_WITHHOST,cep->host,cfgid);
@@ -571,7 +571,7 @@ web_disentangle(vp, ok)
 	       if (vp->proc->vertex != NULL)
 	       vp->proc->vertex->proc = vp->proc;
 	     */
-	    pick_next_vertex(vp->proc, ok);
+	    pick_next_vertex(vp->proc, ok, 0);
 	  }
 	  vp->proc = NULL;
 	}
@@ -743,29 +743,30 @@ struct thread *thr;
 	  if (proc->fed)
 	    proc->overfed += 1;
 	  proc->hungry = 1;	/* Simulate hunger.. */
-	  pick_next_vertex(proc,1);
+	  pick_next_vertex(proc, 1, 0);
 	  /* While we have a thread, and things to feed.. */
 	  while (!proc->fed && proc->thread) {
 	    if (proc->hungry)
 	      feed_child(proc);
 	    if (proc->fed)
 	      proc->overfed += 1;
+	    else
+	      break; /* Huh! Didn't feed it! */
 	    /* See if we should, and can feed more! */
 	    if (proc->thg == NULL ||
 		proc->pid == 0    ||
 		proc->thread == NULL)
 	      break;		/* No new active threads/vertices/proc.. */
-	    if (proc->cmdbuf[0] != 0)
-	      break;		/* it is already overstuffed */
 	    if (proc->overfed >= proc->thg->ce.overfeed)
 	      break;		/* if the limit is zero, don't overfeed ever.*/
 	    /* Ok, increment the counter, and loop back.. */
 	    proc->hungry = 1;	/* Simulate hunger.. */
 	    proc->fed = 1;	/* tell them, it has been fed.. */
-	    pick_next_vertex(proc,1);
+	    pick_next_vertex(proc, 1, 0);
 	    /* If it got next,  ``proc->fed'' is now zero.. */
 	  }
 	  proc->hungry = 0; /* ... satiated.. */
+	  flush_child(proc);
 	  return 1;
 	}
 
@@ -834,9 +835,9 @@ struct thread *thr;
  *   - if all of the threads exhaust, move the process into idle pool
  */
 void
-pick_next_vertex(proc, ok)
+pick_next_vertex(proc, ok, justfree)
 struct procinfo *proc;
-int ok;
+int ok, justfree;
 {
 	struct thread      *thr, *thr0;
 	struct threadgroup *thg;
@@ -844,8 +845,9 @@ int ok;
 
 	thr  = proc->thread;
 
-if (verbose) printf("pick_next_vertex(proc->tofd=%d, thr=0x%p, jobs=%d ok=%d)\n",
-		    proc->tofd, thr, thr ? thr->jobs : 0, ok);
+	if (verbose)
+	  printf("pick_next_vertex(proc->tofd=%d, thr=0x%p, vtx=0x%p, jobs=%d ok=%d justfree=%d)\n",
+		 proc->tofd, thr, proc->vertex, thr ? thr->jobs : 0, ok, justfree);
 
 	if (proc->pid < 0) {	/* "Jim, He is dead!"		*/
 	  if (proc->thread != NULL)
@@ -854,18 +856,23 @@ if (verbose) printf("pick_next_vertex(proc->tofd=%d, thr=0x%p, jobs=%d ok=%d)\n"
 	  if (proc->vertex != NULL)
 	    proc->vertex->proc = NULL;
 	  proc->vertex = NULL;
-if (verbose) printf(" ... NONE, 'Jim, He is dead!'\n");
+	  if (verbose) printf(" ... NONE, 'Jim, He is dead!'\n");
 	  return;
 	}
 
 	if (thr == NULL) {
-if (verbose) printf(" ... NONE, we are idle.\n");
+	  if (verbose) printf(" ... NONE, we are idle.\n");
 	  return; /* WE ARE IDLE! */
 	}
 
-	if (proc->fed == 0) {
-if (verbose) printf(" ... NONE, current one has not been fed..\n");
-	  return; /* Current one has not been fed..	*/
+	/* if (proc->cmdbuf[0] != 0) {
+	   if (verbose) printf(" ... NONE, feeding incomplete..\n");
+	   return;
+	   } */
+
+	if (!justfree && proc->fed == 0 && proc->vertex != NULL) {
+	  if (verbose) printf(" ... NONE, current one has not been fed..\n");
+	  return; /* Current one has not been (completely) fed..	*/
 	}
 
 	thr0 = thr;
@@ -880,7 +887,7 @@ if (verbose) printf(" ... NONE, current one has not been fed..\n");
 	  /* Is the current one in processing ? */
 	  if (vtx->proc == NULL) {
 	    proc->vertex = vtx;
-if (verbose) printf(" ... thr=same vtx=0x%p\n",vtx);
+	    if (verbose) printf(" ... thr=same vtx=0x%p\n",vtx);
 	    proc->fed = 0;
 	    return; /* No, it is eligible! */
 	  }
@@ -893,7 +900,7 @@ if (verbose) printf(" ... thr=same vtx=0x%p\n",vtx);
 	if (proc->overfed > 0) {
 	  /* We are/have overfed, DO NOT CHANGE THREAD YET */
 	  proc->fed = 1;
-if (verbose) printf(" ... overfed=%d, no thread change.\n",proc->overfed);
+	  if (verbose) printf(" ... overfed=%d, no thread change.\n",proc->overfed);
 	  return;
 	}
 
@@ -901,7 +908,7 @@ if (verbose) printf(" ... overfed=%d, no thread change.\n",proc->overfed);
 	/* Pick another thread in our ring */
 	thr = thr->nextthg;
 
-	time(&now);
+	mytime(&now);
 
 	/* the threads are in a ring.. */
 	for ( ;thr != thr0; thr = thr->nextthg) {
@@ -940,9 +947,9 @@ if (verbose) printf(" ... overfed=%d, no thread change.\n",proc->overfed);
 				thread so that at the next time we
 				pick some other thread when skipping
 				failed threads.. */
-if (verbose) printf(" ... thr=0x%p vtx=0x%p ch=%s ho=%s\n",
-		    thr,proc->vertex,
-		    proc->ch->name,proc->ho->name);
+	  if (verbose) printf(" ... thr=0x%p vtx=0x%p ch=%s ho=%s\n",
+			      thr,proc->vertex,
+			      proc->ch->name,proc->ho->name);
 	  proc->fed = 0;
 	  return; /* It is eligible to run! */
 	}
@@ -952,13 +959,13 @@ if (verbose) printf(" ... thr=0x%p vtx=0x%p ch=%s ho=%s\n",
 	     the end of the thread, and wait thread purge to
 	     happen -- by timeouts, or whatever.
 	     We don't idle, we don't move, just return.. */
-if (verbose) printf(" ... OVERFEED - don't change thread yet.\n");
+	  if (verbose) printf(" ... OVERFEED - don't change thread yet.\n");
 	  return;
 	}
 
 	/* No free threads/vertices here, idle the process */
-if (verbose) printf(" ... NONE, idle the process (of=%d, f=%d).\n",
-		    proc->overfed, proc->fed);
+	if (verbose) printf(" ... NONE, idle the process (of=%d, f=%d).\n",
+			    proc->overfed, proc->fed);
 	proc->thread = NULL;
 	proc->vertex = NULL;
 	proc->next = thg->idleproc;
@@ -989,7 +996,7 @@ time_t retrytime;
 		 thr->channel,thr->host,thr->jobs);
 
 	/* find out when to retry */
-	time(&now);
+	mytime(&now);
 
 	/* if we are already scheduled for the future, don't reschedule */
 	if (vtx->wakeup > now) {
@@ -1109,7 +1116,7 @@ reschedule(vp, factor, index)
 #endif
 
 	/* find out when to retry */
-	time(&now);
+	mytime(&now);
 
 	if (verbose)
 	  printf("reschedule %p now %d expiry in %d attempts %d factor %d inum %d (%s/%s: %s)\n",
@@ -1327,7 +1334,7 @@ FILE *fp;
 
 	extern time_t sched_starttime; /* From main() */
 
-	time(&now);
+	mytime(&now);
 
 	if (thrg_root == NULL) {
 	  *timebuf = 0;
@@ -1379,26 +1386,26 @@ FILE *fp;
 		      (thr->vertices->ce_pending == L_CHANNEL ? " channelwait" : " threadwait"));
 	    fprintf(fp,"\n");
 	  }
-	  fprintf(fp,"\t\tThreads: %d",thg->threads);
+	  fprintf(fp,"\t\tThreads: %3d",thg->threads);
 	  if (thg->threads != cnt)
 	    fprintf(fp,"/%d",cnt);
-	  fprintf(fp," Procs: %d",thg->transporters);
-	  if (thg->transporters != procs)
-	    fprintf(fp,"/%d",procs);
-	  fprintf(fp," Plim: %d Flim: %d ",thg->ce.maxkidThreads,thg->ce.overfeed);
-	  fprintf(fp,"Rcpts: %d ",jobsum);
-	  jobtotal  += jobsum;
-	  threadsum += thg->threads;
+	  fprintf(fp," Rcpts: %4d",jobsum);
+	  fprintf(fp," Procs: %2d",thg->transporters);
+	  /* 	  if (thg->transporters != procs)
+		  fprintf(fp,"/%d",procs);		*/
 	  cnt = 0;
 	  for (p = thg->idleproc; p != 0; p = p->next) ++cnt;
-	  fprintf(fp,"Idle: %d",thg->idlecnt);
+	  fprintf(fp," Idle: %2d",thg->idlecnt);
 	  if (thg->idlecnt != cnt)
 	    fprintf(fp,"/%d",cnt);
+	  fprintf(fp," Plim: %2d Flim: %3d ",thg->ce.maxkidThreads,thg->ce.overfeed);
+	  jobtotal  += jobsum;
+	  threadsum += thg->threads;
 	  fprintf(fp,"\n");
 	}
 	*timebuf = 0;
 	saytime((long)(now - sched_starttime), timebuf, 1);
-	fprintf(fp,"Kids: %d   Idle: %d   Msgs: %d   Thrds: %d   Rcpnts: %d   Uptime: %s\n",
+	fprintf(fp,"Kids: %d  Idle: %2d  Msgs: %3d  Thrds: %3d  Rcpnts: %4d  Uptime: %s\n",
 		numkids, idleprocs, global_wrkcnt, threadsum, jobtotal, timebuf);
 }
 
