@@ -24,6 +24,7 @@
 
 #include "prototypes.h"
 
+extern int forkrate_limit;
 
 static int  scheduler_nofiles = -1; /* Will be filled below */
 static int  runcommand __((char **, struct vertex *, struct web *, struct web*));
@@ -48,40 +49,51 @@ int	numkids = 0;
 extern char *getzenv();
 extern char *strchr();
 extern int errno;
-extern time_t time();
 
 
+/* Send "#idle\n" -string to the child.. */
 
 void
 idle_child(proc)
 struct procinfo *proc;
 {
-	/* XX: send "#idle\n" -string to the child.. */
+	int len, rc;
+	/* we are NOT to be called while there is something left in the cmdbuf[] ! */
+	if (proc->cmdlen != 0) {
+	  fprintf(stderr,"idle_child(proc->cmdbuf=\"%s\") -> abort()!\n",proc->cmdbuf);
+	  fflush(stderr);
+	  /* abort(); */ /* Hmm.. ??? */
+	  return;
+	}
+	strcpy(proc->cmdbuf,"#idle\n");
+	len = strlen(proc->cmdbuf);
+	rc = write(proc->tofd, proc->cmdbuf, len);
+	if (rc != len) {
+	  if (rc < 0 && (errno != EAGAIN && errno != EINTR &&
+			 errno != EWOULDBLOCK)) {
+	    /* Some real failure :-( */
+	    pipes_shutdown_child(proc->tofd);
+	    proc->tofd = -1;
+	    proc->vertex->proc = NULL;
+	    proc->thread->proc = NULL;
+	    proc->thread       = NULL;
+	    proc->cmdlen = 0;
+	    return;
+	  }
+	}
+	proc->cmdlen = 0;
 }
 
-void
-feed_child(proc)
+int
+flush_child(proc)
 struct procinfo *proc;
 {
-	struct vertex *vtx;
-	int len, rc;
-
-	/* SIGNAL_HOLD(SIGCHLD); */
-	if (proc->thread == NULL) {
-	  /* SIGNAL_RELEASE(SIGCHLD); */
-	  return; /* Might be called without next process.. */
-	}
-	if (proc->pid <= 0 || proc->tofd < 0) {
-	  /* SIGNAL_RELEASE(SIGCHLD); */
-	  return; /* No process../No write channel.. */
-	}
-
-	if (proc->cmdbuf[0] != 0) {
+	int rc, len;
+	if (proc->cmdlen != 0) {
 	  /* We have some leftovers from previous feed..
 	     .. feed them now.  */
-	  len = strlen(proc->cmdbuf);
-	  rc = write(proc->tofd, proc->cmdbuf, len);
-	  if (rc != len) {
+	  rc = write(proc->tofd, proc->cmdbuf, proc->cmdlen);
+	  if (rc != proc->cmdlen) {
 	    if (rc < 0 && (errno != EAGAIN && errno != EINTR &&
 			   errno != EWOULDBLOCK)) {
 	      /* Some real failure :-( */
@@ -90,75 +102,84 @@ struct procinfo *proc;
 	      proc->vertex->proc = NULL;
 	      proc->thread->proc = NULL;
 	      proc->thread       = NULL;
-	      proc->cmdbuf[0] = 0;
-	      /* SIGNAL_RELEASE(SIGCHLD); */
-	      return;
+	      proc->cmdlen = 0;
+	      return -1;
 	    }
 	    if (rc < 0) rc = 0;
-	    if (rc > 0) {
+	    if (rc > 0)
 	      strcpy(proc->cmdbuf, proc->cmdbuf+rc);
-	    }
+	    proc->cmdlen -= rc;
 	  } else {
-	    proc->cmdbuf[0] = 0; /* Clean the pending info.. */
+	    proc->cmdlen = 0; /* Clean the pending info.. */
 	  }
 	  proc->feedtime = now;
-	  /* SIGNAL_RELEASE(SIGCHLD); */
-	  return; /* We return latter.. */
+	  return proc->cmdlen; /* We return latter.. */
 	}
+	return 0;
+}
 
-	vtx = proc->vertex;
-	vtx->proc = proc;    /* Flag that it is in processing */
-	vtx->ce_pending = 0; /* and clear the pending.. */
+void
+feed_child(proc)
+struct procinfo *proc;
+{
+	struct vertex *vtx;
+	int len, rc;
+	char cmdbuf[500];
 
-	if (proc->thg->withhost)
-	  sprintf(proc->cmdbuf, "%s\n", vtx->cfp->mid);
-	else
-	  sprintf(proc->cmdbuf, "%s\t%s\n", vtx->cfp->mid, proc->ho->name);
-
-	len = strlen(proc->cmdbuf);
-
-	if (verbose)
-	  printf("feed: tofd=%d, fed=%d, len=%d, chan=%s, buf=%s",
-		 proc->tofd,proc->fed,len,proc->ch->name,proc->cmdbuf);
+	if (proc->thread == NULL) {
+	  return; /* Might be called without next process.. */
+	}
+	if (proc->pid <= 0 || proc->tofd < 0) {
+	  return; /* No process../No write channel.. */
+	}
 	if (proc->fed) {
 	  /* DON'T RE-FEED! */
-	  proc->cmdbuf[0] = 0;
 	  if (verbose)
 	    printf(" ... no refeeding\n");
-	  /* SIGNAL_RELEASE(SIGCHLD); */
 	  return;
 	}
 
-	rc = write(proc->tofd, proc->cmdbuf, len);
+	vtx = proc->vertex;
 
-	if (rc != len) {
-	  if (rc < 0 && (errno != EAGAIN && errno != EINTR &&
-			 errno != EWOULDBLOCK)) {
-	    /* Failed ?  How come ?
-	       Close the 'tofd' pipe, and clear proc ptr. */
-	    pipes_shutdown_child(proc->tofd);
-	    proc->tofd = -1;
-	    vtx->proc = NULL;
-	    vtx       = NULL;
-	    proc->thread->proc = NULL;
-	    proc->thread       = NULL;
-	    proc->cmdbuf[0] = 0;
-	  } else {
-	    if (rc < 0) rc = 0; /* Interrupted write,
-				   we return to it latter.. */
-	    strcpy(proc->cmdbuf, proc->cmdbuf+rc);
+	if (proc->thg->withhost)
+	  sprintf(cmdbuf, "%s\n", vtx->cfp->mid);
+	else
+	  sprintf(cmdbuf, "%s\t%s\n", vtx->cfp->mid, proc->ho->name);
+
+	len = strlen(cmdbuf);
+
+	if (len >= (sizeof(proc->cmdbuf) - proc->cmdlen)) {
+	  /* Does not fit there, flush it! */
+	  rc = flush_child(proc);
+	  if (len >= (sizeof(proc->cmdbuf) - proc->cmdlen)) {
+	    /* STILL does not fit there, come back latter.. */
+	    return;
 	  }
-	} else {
-	  if (proc->hungry) --hungry_childs;
-	  proc->cmdbuf[0] = 0; /* Nothing pending */
 	}
-	/* It was fed, clear this flag.. */
+	/* Ok, it does fit in, copy it there.. */
+	strcpy(proc->cmdbuf+proc->cmdlen,cmdbuf);
+	proc->cmdlen += len;
+
+	if (verbose) {
+	  printf("feed: tofd=%d, fed=%d, chan=%s, proc=0x%p, vtx=0x%p, ",
+		 proc->tofd,proc->fed,proc->ch->name, proc, vtx);
+	  fflush(stdout);
+	}
+
+	vtx->proc = proc;    /* Flag that it is in processing */
+	vtx->ce_pending = 0; /* and clear the pending.. */
+	
+	if (proc->hungry) --hungry_childs;
+	/* It was fed (to buffer), clear this flag.. */
 	proc->hungry = 0;
 	proc->fed = 1;
+
+	if (verbose)
+	  printf("len=%d buf=%s", len, cmdbuf);
+
 	proc->feedtime = now;
 	if (vtx)
 	  vtx->attempts += 1; /* We may get it closed above.. */
-	/* SIGNAL_RELEASE(SIGCHLD); */
 }
 
 /*
@@ -173,11 +194,26 @@ start_child(vhead, chwp, howp)
 {
 	char	*av[30], *s, *os, *cp, *ocp, buf[MAXPATHLEN];
 	int	i;
+	static time_t prev_time = 0;
+	static int startcnt = 0; /* How many childs per second (time_t tick..) ? */
+	time_t this_time;
+
 
 	if (verbose)
 	  printf("transport(vhead,chan=%s,host=%s)\n",
 		 chwp->name,howp->name);
 
+	++startcnt;
+	this_time = mytime(NULL);
+	if (this_time != prev_time) {
+	  startcnt = 0;
+	  prev_time = this_time;
+	} else if (startcnt > forkrate_limit) {
+	  if (verbose)
+	    printf(" ... too many forks per second!\n");
+	  return 0;
+	}
+
 	if (vhead->thgrp->ce.argv == NULL) {
 	  fprintf(stderr, "No command defined for %s/%s!\n",
 		  chwp->name, howp->name);
@@ -202,7 +238,12 @@ start_child(vhead, chwp, howp)
 		  ++cp;
 		if (*cp == '}') {
 		  *cp = '\0';
-		  strcpy(s, getzenv(ocp));
+		  if (strcmp(ocp,"host")==0)
+		    strcpy(s,howp->name);
+		  else if (strcmp(ocp,"channel")==0)
+		    strcpy(s,chwp->name);
+		  else
+		    strcpy(s, getzenv(ocp));
 		  s += strlen(s);
 		  *cp = '}';
 		} else
@@ -312,7 +353,7 @@ static void stashprocess(pid, fromfd, tofd, chwp, howp, vhead)
 	memset(proc,0,sizeof(struct procinfo));
 #if 0 /* the memset() does this more efficiently.. */
 	proc->next   = NULL;
-	proc->cmdbuf[0] = 0;
+	proc->cmdlen = 0;
 	proc->reaped = 0;
 	proc->carryover = NULL;
 	proc->hungry = 0;
@@ -332,9 +373,9 @@ static void stashprocess(pid, fromfd, tofd, chwp, howp, vhead)
 	if (howp != NULL) howp->kids += 1;
 	proc->tofd   = tofd;
 	vhead->proc  = proc;
-	time(&proc->hungertime); /* Actually it is not yet 'hungry' as
-				    per reporting so, but we store the
-				    time-stamp anyway */
+	mytime(&proc->hungertime); /* Actually it is not yet 'hungry' as
+				      per reporting so, but we store the
+				      time-stamp anyway */
 	fd_nonblockingmode(fromfd);
 	if (fromfd != tofd)
 	  fd_nonblockingmode(tofd);
@@ -475,6 +516,16 @@ typedef long	fd_mask;
 typedef	struct fd_set { fd_mask	fds_bits[1]; } fd_set;
 #endif	/* notdef */
 
+#ifdef _AIX
+#include <sys/types.h>
+#include <sys/select.h>
+/*	struct {
+		fd_set  fdsmask;
+		fd_set  msgsmask;
+	} readfds, writefds;
+*/
+#endif
+
 #ifndef	FD_SET
 #define	FD_SET(n, p)	((p)->fds_bits[0] |= (1 << (n)))
 #define	FD_CLR(n, p)	((p)->fds_bits[0] &= ~(1 << (n)))
@@ -483,13 +534,7 @@ typedef	struct fd_set { fd_mask	fds_bits[1]; } fd_set;
 #endif	/* !FD_SET */
 #endif	/* !NFDBITS */
 
-#ifdef _AIX
-#include <sys/types.h>
-#include <sys/select.h>
-	struct {
-		fd_set  fdsmask;
-		fd_set  msgsmask;
-	} readfds, writefds;
+#if defined(_AIX) && !defined(FD_SET) /* Pre AIX 4.1 ? */
 #define _FD_SET(sock,var) FD_SET(sock,&var.fdsmask)
 #define _FD_CLR(sock,var) FD_CLR(sock,&var.fdsmask)
 #define _FD_ZERO(var) FD_ZERO(&var.fdsmask)
@@ -527,7 +572,7 @@ time_t timeout;
 	if (in_select) fprintf(stderr,"**** recursed into mux()! ***\n");
 	in_select = 1;
 
-	time(&now);
+	mytime(&now);
 
 	tv.tv_sec = timeout - now; /* Timeout in seconds */
 	if (timeout < now)
@@ -542,7 +587,7 @@ time_t timeout;
 	    if (cpids[i].pid != 0) {
 	      _FD_SET(i, rdmask);
 	      maxf = i;
-	      if (cpids[i].cmdbuf[0] != 0)
+	      if (cpids[i].cmdlen != 0)
 		_FD_SET(i, wrmask);
 	    }
 	if (querysocket >= 0) {
@@ -622,9 +667,9 @@ time_t timeout;
 	    /* In case we have non-completed 'feeds', try feeding them */
 	    for (i = 0; i < scheduler_nofiles; ++i)
 	      if (cpids[i].pid > 0
-		  && cpids[i].cmdbuf[0] != 0
+		  && cpids[i].cmdlen != 0
 		  && _FD_ISSET(i, wrmask)) {
-		feed_child(&cpids[i]);
+		flush_child(&cpids[i]);
 		/* Because this loop might take a while ... */
 		queryipccheck();
 	      }
@@ -683,7 +728,7 @@ queryipcinit()
 
 	if (querysocket >= 0)
 		return;
-	time(&now);
+	mytime(&now);
 	if ((serv = getservbyname("mailq", "tcp")) == NULL) {
 	  fprintf(stderr, "No 'mailq' tcp service defined!\n");
 	  /* try again in 5 minutes or so */
@@ -734,7 +779,7 @@ time_t timeout;
 	    if (cpids[fd].pid != 0)
 	      readfrom(fd);
 
-	time(&now);
+	mytime(&now);
 	if (timeout > now)
 	  sleep(1);
 	return 1;
