@@ -5,7 +5,7 @@
  */
 /*
  *	Lots of modifications (new guts, more or less..) by
- *	Matti Aarnio <mea@nic.funet.fi>  (copyright) 1992-1995
+ *	Matti Aarnio <mea@nic.funet.fi>  (copyright) 1992-1996
  */
 
 #include "hostenv.h"
@@ -31,7 +31,6 @@ extern int ermdir();
 extern int rmdir();
 extern int eunlink();
 extern struct spblk *sp_fhead(), *sp_fnext();
-extern time_t time();
 extern time_t now;
 extern int global_wrkcnt;
 
@@ -40,6 +39,7 @@ extern int global_wrkcnt;
  * necessary to update our model of the world to reflect the new reality.
  *
  *	#hungry
+ *	#resync 18453-321
  *	# other debug comment
  *	18453/3527\tIETF-NOTARY\tok Quick SMTP connection!
  *	18453/3527\tIETF-NOTARY\tdeferred Unable to contact host!
@@ -84,7 +84,7 @@ update(fd, diagnostic)
 	  if (proc->tofd >= 0 &&
 	      proc->hungry == 0) {
 	    proc->hungry = 1;
-	    time(&now);
+	    mytime(&now);
 	    proc->hungertime = now;
 	    ++hungry_childs;
 	  }
@@ -92,19 +92,20 @@ update(fd, diagnostic)
 	  /* .. propably not a good idea to try to pick any next.. */
 	  if (proc->overfed > 0)
 	    proc->overfed -= 1;
-	  pick_next_vertex(proc, 1);
+	  pick_next_vertex(proc, 1, 0);
 	  if (proc->hungry)
 	    feed_child(proc);
+	  flush_child(proc);
 	  return;
 	}
 	if (*diagnostic == '#' /* DEBUG comment.. */) {
 	  if (strncmp(diagnostic,"#hungry",7)==0) {
 	    /* This is an "actor model" behaviour,
 	       where actor tells, when it needs a new
-	       job fed to it. */
+	       job to be fed to it. */
 	    if (proc->tofd   >= 0) {
 	      proc->hungry = 1;
-	      time(&proc->hungertime);
+	      mytime(&proc->hungertime);
 	      if (proc->overfed > 0)
 		/* It was overfed, decrement that counter first.. */
 		proc->overfed -= 1;
@@ -112,28 +113,29 @@ update(fd, diagnostic)
 		++hungry_childs;
 		/* Unless it is still overfed,
 		   Pick next, and feed it! */
-		pick_next_vertex(proc,1);
+		pick_next_vertex(proc, 1, 0);
 		/* While we have a thread, and things to feed.. */
 		while (!proc->fed && proc->thread) {
 		  if (proc->hungry)
 		    feed_child(proc);
 		  if (proc->fed)
 		    proc->overfed += 1;
+		  else
+		    break; /* Huh! Feed/flush failure! */
 		  /* See if we should, and can feed more! */
 		  if (proc->thg == NULL ||
 		      proc->pid == 0    ||
 		      proc->thread == NULL)
 		    break;	/* No new active threads/vertices/proc.. */
-		  if (proc->cmdbuf[0] != 0)
-		    break;	/* it is already overstuffed */
 		  if (proc->overfed >= proc->thg->ce.overfeed)
 		    break;	/* if the limit is zero, don't overfeed ever.*/
 		  /* Ok, increment the counter, and loop back.. */
 		  proc->hungry = 1; /* Simulate hunger.. */
 		  proc->fed = 1; /* tell them, it has been fed.. */
-		  pick_next_vertex(proc,1);
+		  pick_next_vertex(proc, 1, 0);
 		  /* If it got next,  ``proc->fed'' is now zero.. */
 		}
+		flush_child(proc);
 		proc->hungry = 0; /* ... satiated.. */
 	      } else {
 		if (verbose)
@@ -234,7 +236,6 @@ unctlfile(cfp, no_unlink)
 	  syslog(LOG_INFO, "%s: done", cfp->mid);
 #endif	/* LOG_INFO */
 	  eunlink(cfp->mid);
-	  --global_wrkcnt;
 	  if (verbose)
 	    printf("%s: unlink %s (mid=0x%p)", cfp->logident, cfp->mid, cfp->mid);
 
@@ -254,22 +255,34 @@ unctlfile(cfp, no_unlink)
 	if (spl != NULL)
 	  sp_delete(spl, spt_mesh[L_CTLFILE]);
 	free_cfp_memory(cfp);
+	--global_wrkcnt;
 }
 
+/* unvertex() .. the ``vp'' CAN'T be invalid.. */
 void unvertex(vp, justfree, ok)
 	register struct vertex *vp;
 	int justfree, ok;
 {
 	int	i, remove;
+	int	reset_proc = 0;
 
 	if (vp->ngroup > 0)
 	  return;
 
+if (verbose && justfree < 0) fprintf(stderr,"unvertex(vtx=0x%p (thr=0x%p proc=0x%p, ng=%d) ,%d,%d)\n",vp,vp->thread,vp->proc,vp->ngroup,justfree,ok);
+
+	if (vp->thread->vertices == vp && vp->thread->proc &&
+	    vp->thread->proc->vertex == vp) {
+	    /* Whoops, we are the active first on thread.. */
+	  if (vp->proc == NULL)
+	    vp->proc = vp->thread->proc;
+	  vp->proc->vertex = vp;
+	}
 	if (vp->proc && vp->proc->vertex == vp) {
 	  vp->proc->fed = 1;     /* Mark it fed just in case.. */
 	  vp->proc->overfed = 0; /* .. and clear this .. */
 	  /* Pick next, but don't feed it (yet)! */
-	  pick_next_vertex(vp->proc, ok);
+	  pick_next_vertex(vp->proc, ok, justfree);
 	  if (vp->proc && vp->proc->vertex == vp){
 	    fprintf(stderr,
 		    "unvertex(vtx=0x%p,%d,%d) failed to pick_next_vertex()!\n",
@@ -277,6 +290,11 @@ void unvertex(vp, justfree, ok)
 	    abort();
 	  }
 	}
+	if (vp->thread->proc && vp->thread->proc->vertex == vp) {
+	  /* XX: This is actually vestigal from somewhere, and
+	         should not occur at all.. */
+	  vp->thread->proc->vertex = NULL;
+	}
 
 	for (i = 0; i < SIZE_L; ++i) {
 	  if (vp->next[i] != NULL)
@@ -446,6 +464,14 @@ saytime(value, buf, shortform)
 	  value = -value;
 	}
 
+	if (value == 0) {
+	  if (shortform)
+	    strcpy(p,"0s");
+	  else
+	    strcpy(p,"0 sec");
+	  return buf;
+	}
+
 	secs = value % 60;
 	value /= 60;
 	mins = value % 60;
@@ -527,7 +553,7 @@ static void logstat(fp,vp,reason)
 	struct vertex *vp;
 	char *reason;
 {
-	time(&now);
+	mytime(&now);
 	fprintf(fp,"%ld %s %ld %ld %s %s/%s\n",
 		(long)vp->cfp->ctime, vp->cfp->mid,
 		(long)(vp->cfp->envctime - vp->cfp->ctime),
@@ -707,7 +733,7 @@ static int u_retryat(vp, index, inum, offset, notary, message)
 	char	*cp;
 
 	if (*message == '+') {
-	  time(&retrytime);
+	  mytime(&retrytime);
 	  ++message;
 	} else
 	  retrytime = 0;
@@ -744,7 +770,7 @@ static int u_retryat(vp, index, inum, offset, notary, message)
 	/* ``vp'' might become expired by  thread_reschedule() .. */
 	if (vp->proc && vp->proc->vertex == vp)
 	  /* Pick next, but don't feed it (yet)! */
-	  pick_next_vertex(vp->proc, 0);
+	  pick_next_vertex(vp->proc, 0, 0);
 
 	thread_reschedule(vp->thread, retrytime, index);
 
