@@ -455,11 +455,15 @@ int *allnump;	/* Return a flag about all-numeric field.. */
 
 char *rfc821_domain(s,strict)		/* "#" <number> | "[" <dotnum> "]" | <name> | <name> "." <domain> */
 char *s;
+int strict;
 {
 	char *p = s, *q;
 	int allnum;	/* If all fields are numeric, it isn't domain.. */
 
-	if (!s || !*s) return s; /* Pathological termination */
+	if (!s || !*s) {
+	  rfc821_error = "RFC821: No input";
+	  return s; /* Pathological termination */
+	}
 
 	if (*p == '[') {
 	  q = rfc821_dotnum(p+1,strict);
@@ -488,13 +492,19 @@ char *s;
 
 	while (p && q > p && *q == '.') {
 	  p = q+1;
+	  if (*p == 0 || *p == '>') {
+	    rfc821_error = "Spurious dot (.) at the end of the domain name";
+	    rfc821_error_ptr = q;
+	    return s;
+	  }
 	  q  = rfc821_name(p,strict,&allnum);
 	}
 	if (allnum) {
 	  rfc821_error = "Should this be of <dotnum> format ? ( [nn.nn.nn.nn] )";
 	  return s;
 	}
-	/*rfc821_error = "bad syntax on domain";*/
+	if (!rfc821_error)
+	  rfc821_error = "bad syntax on domain";
 	if (!q || p == q) return s;	/* Report whatever  <name>  reports */
 	return q;	/* Ok */
 }
@@ -502,6 +512,7 @@ char *s;
 static
 char *rfc821_mailbox(s,strict)	/* <local-part> "@" <domain> */
 char *s;		/* Report error */
+int strict;
 {
 	char *p = s, *q;
 
@@ -530,8 +541,7 @@ char *s;		/* Report error */
 	++p;
 	q = rfc821_domain(p,strict);
 	if (q == p) {
-	  rfc821_error_ptr = s;
-	  rfc821_error = "Missing domain from mailbox definition";
+	  /* Error report from domain.. */
 	  return s;
 	}
 	return q;
@@ -553,6 +563,7 @@ char *s;
 static
 char *rfc821_string(s,strict)
 char *s;
+int strict;
 {
 	char *p = s, *q;
 
@@ -575,6 +586,7 @@ char *s;
 static
 char *rfc821_dot_string(s,strict)
 char *s;
+int strict;
 {
 	char *p = s, *q;
 
@@ -593,6 +605,7 @@ char *s;
 static
 char *rfc821_qtext(s,strict)
 char *s;
+int strict;
 {
 	char *p = s;
 	int fail = 0;
@@ -625,6 +638,7 @@ char *s;
 static
 char *rfc821_quoted_string(s,strict)
 char *s;
+int strict;
 {
 	char *p = s, *q;
 
@@ -652,6 +666,7 @@ char *s;
 static
 char *rfc821_localpart(s,strict)/* <dot-string> | <quoted-string> */
 char *s;			/* Stretched RFC821 a bit here.. !%-hacks */
+int strict;
 {
 	char *p = s, *q;
 
@@ -679,6 +694,7 @@ char *s;			/* Stretched RFC821 a bit here.. !%-hacks */
 
 char *rfc821_path2(s,strict)	/*  [ <a-d-l> ":" ] <mailbox>  */
 char *s;
+int strict;
 {
 	char *p = s, *q;
 
@@ -707,6 +723,7 @@ char *s;
 
 char *rfc821_path(s,strict)	/*  "<" [ <a-d-l> ":" ] <mailbox> ">"  */
 char *s;
+int strict;
 {
 	char *p = s, *q;
 
