@@ -83,7 +83,7 @@ char *Copyright2= "Copyright 1991-1995 Matti Aarnio";
 #endif	/* SIGCHLD */
 
 #ifndef	MAXHOSTNAMELEN
-#define	MAXHOSTNAMELEN 64
+#define	MAXHOSTNAMELEN 256
 #endif	/* MAXHOSTNAMELEN */
 
 /*
@@ -216,7 +216,7 @@ unsigned short	auth_flag = 0;
 # ifndef	AUTH_TIMEOUT
 #  define	AUTH_TIMEOUT	5
 # endif	/* AUTH_TIMEOUT */
-char		auth_username[MAXHOSTNAMELEN+1];
+char		auth_username[BUFSIZ]; /* It can be mighty big.. */
 static void	setrfc1413auth();
 #endif	/* USE_AUTH */
 
@@ -291,7 +291,7 @@ int insecure;
 					progname, logfile, strerror(errno));
 		} else {
 			logfp = fdopen(fd, "a");
-/* XX: */		setbuf(logfp,NULL);
+/* XX: */		setvbuf(logfp,NULL,_IOLBF,BUFSIZ); /* Line-buffered */
 		}
 	} else
 		logfp = NULL;
