@@ -51,6 +51,9 @@ extern char *mydomain();
 #define	MAXPATHLEN 1024
 #endif	/* MAXPATHLEN */
 
+int D_alloc = 0; /* For tmalloc() from libz.a ... */
+
+
 int
 main(argc, argv)
 	int argc;
@@ -126,11 +129,15 @@ main(argc, argv)
 	  if (dp != NULL) {
 	    process(dp);
 	    ctlclose(dp);
+	  } else {
+	    printf("#resync %s\n",msgfilename);
+	    fflush(stdout);
 	  }
 	}
 	return 0;
 }
 
+/*
 #ifndef MALLOC_TRACE
 univptr_t
 tmalloc(n)
@@ -139,6 +146,35 @@ tmalloc(n)
 	return emalloc((u_int)n);
 }
 #endif
+*/
+
+/* Pick recipient address from the input line.
+   EXTREMELY Simple minded parsing.. */
+static void pick_env_addr(buf,mfp)
+char *buf;
+FILE *mfp;
+{
+	char *s = buf;
+
+	while (*s != 0 && *s != ' ' && *s != '\t' && *s != ':') ++s;
+	if (*s != ':') return; /* BAD INPUT! */
+	buf = ++s; /* We have skipped the initial header.. */
+
+	s = strchr(buf,'\n');
+	if (s) *s = 0;
+	s = strchr(buf,'<');
+	if (s != NULL) {
+	  /*  Cc:  The Postoffice managers <postoffice> */
+	  buf = ++s;
+	  s = strrchr(buf,'>');
+	  if (s) *s = 0;
+	  else return; /* No trailing '>' ? BAD BAD! */
+	  fprintf(mfp,"to %s\n",buf);
+	} else {
+	  /*  Cc: some-address  */
+	  fprintf(mfp,"to %s\n",buf);
+	}
+}
 
 void
 process(dp)
@@ -152,6 +188,10 @@ process(dp)
 	char *boundarystr = buildboundary("errmail");
 	char lastchar;
 	int reportcnt = 0;
+	struct stat stbuf;
+
+	if (fstat(dp->msgfd, &stbuf) != 0)
+	  abort(); /* This is a "CAN'T FAIL" case.. */
 
 	/* recipient host field is the error message file name in FORMSDIR */
 	/* recipient user field is the address causing the error */
@@ -181,9 +221,6 @@ process(dp)
 
 	fprintf(mfp, "to %s\n",rp->addr->link->user);
 
-	/* copy To: from error return address */
-	fprintf(mfp, "To: <%s>\n", rp->addr->link->user);
-
 	/* copy error message file itself */
 	if ((mailshare = getzenv("MAILSHARE")) == NULL)
 	  mailshare = MAILSHARE;
@@ -195,9 +232,27 @@ process(dp)
 	if ((efp = fopen(mfpath, "r")) != NULL) {
 	  int inhdr = 1;
 	  buf[sizeof(buf)-1] = 0;
+	  while (fgets(buf,sizeof(buf)-1,efp) != NULL) {
+	    if (strncmp(buf,"HDR",3)==0)
+	      continue;
+	    else if (strncmp(buf,"ADR",3)==0)
+	      pick_env_addr(buf+4,mfp);
+	    else if (strncmp(buf,"SUB",3)==0)
+	      continue;
+	    else
+	      break;
+	  }
+	  fseek(efp,0,0); /* Rewind! */
+	  fputs("env-end\n",mfp);
+
+	  /* copy To: from error return address */
+	  fprintf(mfp, "To: <%s>\n", rp->addr->link->user);
+
 	  while (fgets(buf,sizeof(buf)-1,efp) != NULL) {
 	    if (strncmp(buf,"HDR",3)==0) {
 	      fputs(buf+4,mfp);
+	    } else if (strncmp(buf,"ADR",3)==0) {
+	      fputs(buf+4,mfp);
 	    } else if (strncmp(buf,"SUB",3)==0) {
 	      fputs(buf+4,mfp);
 	    } else {
@@ -246,20 +301,36 @@ process(dp)
 
 	/* Print out errors in IETF-NOTARY format as well! */
 
-	fprintf(mfp, "Original-MTS-Type: INET\n");
+	if (mydomain() != NULL) {
+	  fprintf(mfp, "Reporting-MTA: dns; %s\n", mydomain() );
+	} else {
+	  fprintf(mfp, "Reporting-MTA: x-local-hostname; -unknown-\n");
+	}
 	if (dp->envid != NULL)
 	  fprintf(mfp, "Original-Envelope-Id: %s\n",dp->envid);
-	fprintf(mfp, "Final-MTS-Type: INET\n");
-	fprintf(mfp, "Final-MTA: %s\n\n", mydomain() );
+	/* rfc822date() returns a string with trailing newline! */
+	fprintf(mfp, "Arrival-Date: %s", rfc822date(&stbuf.st_ctime));
+	fprintf(mfp, "\n");
 
 	for (rp = dp->recipients; rp != NULL; rp = rp->next) {
 	  /* If not prohibited, print it! */
+	  const char *typetag;
+	  char *rcpt;
+	  static char const *type_rfc   = "RFC822";
+	  static char const *type_local = "X-LOCAL";
+
 	  if ((rp->dsnflags & _DSN_NOTIFY_NEVER) != 0)
 	    continue;
-
-	  fprintf(mfp, "Rcpt: %s\n", rp->addr->user);
+	  rcpt = rp->addr->user;
+	  if (strchr(rcpt,'@') != NULL) {
+	    typetag = type_rfc;
+	    if (strncmp(rcpt,"ns:",3)==0) /* 'hold'-channel stuff */
+	      typetag = type_local;
+	  } else
+	    typetag = type_local;
+	  fprintf(mfp, "Final-Recipient: %s; %s\n", typetag, rcpt);
 	  fprintf(mfp, "Action: failed\n");
-	  fprintf(mfp, "Status: 500 (%s)\n", rp->addr->host );
+	  fprintf(mfp, "Diagnostic-Code: X-LOCAL; 500 (%s)\n", rp->addr->host );
 	  if (rp->orcpt != NULL)
 	    fprintf(mfp, "Original-Rcpt: %s\n",rp->orcpt);
 	  fprintf(mfp, "\n");
