@@ -136,6 +136,8 @@ extern int ranny();
 #define	putc	fputc
 #endif	/* lint */
 
+FILE *verboselog = NULL; /* getrr() needs this .. */
+
 int
 main(argc, argv)
 	int argc;
@@ -215,6 +217,9 @@ main(argc, argv)
 	  if (dp != NULL) {
 	    process(dp);
 	    ctlclose(dp);
+	  } else {
+	    printf("#resync %s\n",file);
+	    fflush(stdout);
 	  }
 	}
 	exit(0);
