@@ -96,6 +96,7 @@ typedef union {
 } querybuf;
 
 extern int h_errno;
+extern FILE *verboselog;
 
 int
 getrr(host, hbsize, rrtype, cnamelevel)	/* getrrtype with completion */
@@ -217,18 +218,25 @@ getrrtype(host, hbsize, rrtype, cnamelevel)
 	    /* If we have seen a WKS, it had better have SMTP,
 	     * however in absence of a WKS, assume SMTP.
 	     */
-	    if (n < sizeof (u_long) + 1) {
-	      cp = nextcp;	/* ? */
+	    if (n <= 4) {	/* IP address! */
+	      cp = nextcp;	/* bad data.. */
 	      continue;
 	    }
 	    ok = 0;
-	    cp += sizeof(u_long); /* skip IP address */
+	    cp += 4;		/* skip the IP address */
 	    if (*cp++ == IPPROTO_TCP) {	/* check protocol */
 	      if (cp + (IPPORT_SMTP/8) < nextcp
 		  && (*(cp+(IPPORT_SMTP/8))
-		      & (0x80>>IPPORT_SMTP%8)))
+		      & (0x80>>IPPORT_SMTP%8))) {
+		if (verboselog)
+		  fprintf(verboselog,"WKS: Found TCP/SMTP\n");
 		return 1;
-	    }
+	      }
+	      if (verboselog)
+		fprintf(verboselog,"WKS: Found WKS of TCP, but not SMTP\n");
+	    } else
+	      if (verboselog)
+		fprintf(verboselog,"WKS: Found WKS other protocol, than TCP\n");
 	    cp = nextcp;
 	    continue;
 	  } else {
