@@ -1,6 +1,9 @@
 /*
  *	Copyright 1988 by Rayan S. Zachariassen, all rights reserved.
  *	This will be free software, but only when it is finished.
+ *
+ *  FCNTL-style locking by Matti Aarnio <mea@nic.funet.fi>
+ *  Theory:  On system where it works, it makes LESS IO
  */
 
 /*
@@ -28,6 +31,32 @@ lockaddr(fd, map, offset, was, new)
 	char	lockbuf[1+_CFTAG_RCPTPIDSIZE+1];
 	int	newlock = 0;
 
+#ifdef USE_FCNTLLOCK
+	struct flock fl;
+	int rc = 0;
+
+	fl.l_type   = F_WRLCK;
+	fl.l_start  = offset;
+	fl.l_whence = SEEK_SET;
+	fl.l_len    = 10; /* fixed.. */
+
+	if (new == _CFTAG_LOCK || new == _CFTAG_DEFER) {
+	  rc = fcntl(fd,F_GETLK,&fl);
+	  if (rc == -1) {
+	    warning("lockaddr: fcntl() lock error");
+	    return 0;
+	  }
+	}
+	lockbuf[1] = ' ';
+	if (was == _CFTAG_NORMAL) {
+	  if (fl.l_type == F_UNLCK)
+	    lockbuf[0] = was;
+	  else
+	    lockbuf[0] = _CFTAG_LOCK;
+	} else if (was == _CFTAG_LOCK) {
+	  if (fl.l_type == F_UNLCK)
+	}
+#endif
 	if (map) {
 	  /* MMAP()ed block helps.. */
 	  memcpy(lockbuf,map+offset,sizeof(lockbuf));
@@ -51,6 +80,9 @@ lockaddr(fd, map, offset, was, new)
 	  lockbuf[0] = new;
 	  if (newlock) {
 	    if (new == _CFTAG_LOCK) {
+#ifdef USE_FCNTLLOCK
+	      /* Using FCNTL region locking */
+#else
 	      /* Mark the lock with client process-id */
 	      sprintf(lockbuf+1,"%*d",_CFTAG_RCPTPIDSIZE,getpid());
 	      if (map)
@@ -58,6 +90,19 @@ lockaddr(fd, map, offset, was, new)
 	      else if (write(fd,lockbuf,
 			     _CFTAG_RCPTPIDSIZE+1) != _CFTAG_RCPTPIDSIZE+1)
 		return 0;
+#endif
+	    } else if (new == _CFTAG_DEFER) {
+#ifdef USE_FCNTLLOCK
+	      /* Using FCNTL region locking */
+#else
+	      /* Clear the lock location */
+	      sprintf(lockbuf+1,"%*s",_CFTAG_RCPTPIDSIZE,"");
+	      if (map)
+		memcpy(map+offset, lockbuf, _CFTAG_RCPTPIDSIZE+1);
+	      else if (write(fd,lockbuf,
+			     _CFTAG_RCPTPIDSIZE+1) != _CFTAG_RCPTPIDSIZE+1)
+		return 0;
+#endif
 	    } else {
 	      /* Clear the lock location */
 	      sprintf(lockbuf+1,"%*s",_CFTAG_RCPTPIDSIZE,"");
@@ -73,4 +118,3 @@ lockaddr(fd, map, offset, was, new)
 	warning("lockaddr: expected '%c' found '%c'\n", was, new);
 	return 0;
 }
-
