@@ -548,8 +548,8 @@ struct rcpt *rp;
 	int cte = 0;
 	int mime = 0;
 
-	if (*(rp->newmsgheadercvt) != NULL)
-	  hdrs = *(rp->newmsgheadercvt);
+	/* if (*(rp->newmsgheadercvt) != NULL)
+	   hdrs = *(rp->newmsgheadercvt); */ /* here we check the ORIGINAL headers.. */
 
 	if (!hdrs) return 0;
 
