@@ -586,7 +586,7 @@ deliver(dp, rp, timestring)
 	char *timestring;
 {
 	register char **maild;
-	int fdmail, messagefd;
+	int fdmail;
 	uid_t uid;
 	FILE *fp;
 	int hasdir;
@@ -605,7 +605,6 @@ deliver(dp, rp, timestring)
 	char *maillockuser;
 #endif
 
-	messagefd = dp->msgfd;
 	uid = atol(rp->addr->misc);
 	at = strchr(rp->addr->user,'@');
 	switch (*(rp->addr->user)) {
@@ -1010,6 +1009,7 @@ deliver(dp, rp, timestring)
 	alarm(0);
 
 
+	/* Where is the end of file -- before we write anything */
 	eofindex = lseek(fdmail, 0L, L_XTND);
 #if	defined(BIFF) || defined(RBIFF)
 	if (nbp != NULL)
@@ -1130,7 +1130,8 @@ putmail(dp, rp, fdmail, fdopmode, timestring, file)
 		ftruncate(FILENO(fp), (u_long)eofindex);
 #endif /* USE_FTRUNCATE */
 	      fclose(fp);
-	      return NULL;
+	      fp = NULL;
+	      goto time_reset;
 	    }
 	  }
 	} else if (eofindex > 0L && eofindex < (sizeof "From x\n")) {
@@ -1164,7 +1165,8 @@ putmail(dp, rp, fdmail, fdopmode, timestring, file)
 	  if (eofindex >= 0)
 	    ftruncate(FILENO(fp), (u_long)eofindex);
 	  fclose(fp);
-	  return NULL;
+	  fp = NULL;
+	  goto time_reset;
 	}
 	lastch = appendlet(dp, rp, fp, file, is_mime);
 
@@ -1174,7 +1176,8 @@ putmail(dp, rp, fdmail, fdopmode, timestring, file)
 	  if (eofindex >= 0)
 	    ftruncate(FILENO(fp), (u_long)eofindex);
 	  fclose(fp);
-	  return NULL;
+	  fp = NULL;
+	  goto time_reset;
 	}
 
 	/* End of the file, and MMDF-style ? */
@@ -1196,8 +1199,16 @@ putmail(dp, rp, fdmail, fdopmode, timestring, file)
 	  if (eofindex >= 0)
 	    ftruncate(FILENO(fp), (u_long)eofindex);
 	  fclose(fp);
-	  return NULL;
+	  fp = NULL;
+	  goto time_reset;
 	}
+
+	/* This is actually Linux-thing, we reset the last access time
+	   to be that of before we did read the file, last modification
+	   into current moment -- At Linux 1.3.8x the system is a bit
+	   over-eager to update atime information at  O_RDWR -file.. */
+	time(&st.st_mtime); /* Set the modification time to be now.. */
+
 	if (logfp != NULL) {
 	  /* [haa@cs.hut.fi] added more info here to catch errors
 	     in delivery  */
@@ -1214,6 +1225,24 @@ putmail(dp, rp, fdmail, fdopmode, timestring, file)
 	  fflush(logfp);
 	}
 
+      time_reset:
+
+	if (st.st_atime != 0) {
+#ifdef	USE_TIMEVAL
+	  struct timeval tv[2];
+	  tv[0].tv_sec = st.st_atime;
+	  tv[1].tv_sec = st.st_mtime;
+	  tv[0].tv_usec = tv[1].tv_usec = 0;
+	  rc = utimes(file, tv);
+#else  /* !USE_TIMEVAL */
+	  struct utimbuf tv;
+	  tv.actime  = st.st_atime;
+	  tv.modtime = st.st_mtime;
+	  rc = utime(file, &tv);
+#endif /* USE_TIMEVAL */
+	}
+
+
 	return fp;
 }
 
@@ -1730,7 +1759,7 @@ appendlet(dp, rp, fp, file, ismime)
 	    readalready = 0;
 	}
 #else /* USE_MMAP  --  get the input from  mmap():ed memory area.. */
-	char *s;
+	char *s; /* message body start */
 	s = dp->let_buffer + dp->msgbodyoffset;
 	if (ismime) {
 	  int i;
@@ -1938,9 +1967,6 @@ writebuf(fp, buf, len, lastchp)
 	static char frombuf[8];
 	static char *fromp = NULL;
 
-	static char *buf2 = NULL;
-	static int buf2len = 0;
-
 	if (buf == NULL) {	/* magic initialization */
 	  save = mmdf_mode ? 0 : 'F';
 	  frombuf[0] = 0;
@@ -1948,20 +1974,6 @@ writebuf(fp, buf, len, lastchp)
 	  return 0;
 	}
 
-#ifdef	USE_ALLOCA
-	buf2 = alloca(len+1);
-#else
-	if (buf2 == NULL) {
-	  buf2 = emalloc(len+1);
-	  buf2len = len;
-	}
-	if (buf2len < len) {
-	  free(buf2);
-	  buf2 = emalloc(len+1);
-	  buf2len = len;
-	}
-#endif
-
 	/* -------------------------------------------- */
 	/* Non-MIME processing				*/
 
@@ -1990,7 +2002,7 @@ writebuf(fp, buf, len, lastchp)
 		case 'm':	expect = ' '; break;
 		case ' ':
 		  /* Write the separator, and the word.. */
-		  if (fwrite(">From ", 6, 1, fp) == 0)
+		  if (fwrite(">From ", 6, 1, fp) < 0 || ferror(fp))
 		    { tlen = -1; break; }
 		  /* anticipate future instances */
 		  expect = '\0';
@@ -2106,7 +2118,8 @@ writemimeline(fp, buf, len, lastchp)
 	  }
 	  /* Uhh... Stored the conversion result into buf2[], length  tlen */
 	  if (!mmdf_mode && buf2[0] == 'F' && strncmp(buf2,"From ",5)==0)
-	    putc('>',fp);
+	    if (putc('>',fp) == EOF)
+	      return -1;
 	  if (tlen > 0) {
 	    i = fwrite(buf2, 1, tlen, fp);
 	    *lastchp = buf2[tlen-1];
