@@ -224,7 +224,7 @@ main(argc, argv)
 	  ctlsticky((char *)NULL, (char *)NULL); /* reset */
 	  dp = ctlopen(file, channel, host, &getout, ctlsticky, NULL);
 	  if (dp == NULL) {
-	    putc('\n',stdout);
+	    printf("#resync %s\n",file);
 	    fflush(stdout);
 	    continue;
 	  }
