@@ -1,7 +1,7 @@
 /*
  *	Copyright 1988 by Rayan S. Zachariassen, all rights reserved.
  *	This will be free software, but only when it is finished.
- *	Copyright 1991-1994 by Matti Aarnio -- modifications, including MIME
+ *	Copyright 1991-1996 by Matti Aarnio -- modifications, including MIME
  */
 
 #define	RFC974		/* If BIND, check that TCP SMTP service is enabled */
@@ -203,6 +203,7 @@ int first_uid = 0;		/* Make the opening connect with the UID of the
 				   "nobody", in which case use "daemon"      */
 
 int D_alloc = 0;		/* Memory usage debug */
+int no_pipelining = 0;		/* In case the system just doesn't cope with it */
 
 static const char FAILED[] = "failed";
 
@@ -296,7 +297,7 @@ int smtp_isopen = 0;
 
 static char *logtag()
 {
-	static char buf[20];
+	static char buf[30];
 	static int logcnt = 0;
 	static char id = 0;
 
@@ -313,20 +314,24 @@ static char *logtag()
 	  if (id > 'Z') id += ('a'-'Z'+1);
 	}
 
-	sprintf(buf,"%05d%c%05d",pid,id,logcnt);
+	time(&now);
+
+	sprintf(buf,"%05d%c%05d%05d",pid,id,logcnt,now % 100000);
 	++logcnt;
 	return buf;
 }
 
 int
-main(argc, argv)
+main(argc, argv, envp)
 	int argc;
 	char *argv[];
+	char *envp[];
 {
 	char file[MAXPATHLEN+128];
 	char *channel = NULL, *host = NULL;
 	int i, fd, errflg, c, smtpstatus;
 	int need_host = 0;
+	int idle;
 	FILE *smtpfp[2];
 	struct ctldesc *dp;
 #ifdef	BIND
@@ -343,8 +348,12 @@ main(argc, argv)
 	getout = 0;
 	cmdline = &argv[0][0];
 	eocmdline = cmdline;
+
 	for (i = 0; i < argc; ++i)
 	  eocmdline += strlen(argv[i]) + 1;
+	/* Can overwrite also the environment strings.. */
+	for (i = 0; envp[i] != NULL; ++i)
+	  eocmdline += strlen(envp[i]) + 1;
 
 	SIGNAL_HANDLESAVE(SIGINT, SIG_IGN, oldsig);
 	if (oldsig != SIG_IGN)
@@ -372,7 +381,7 @@ main(argc, argv)
 	myhostname[0] = '\0';
 	remotemsg[0] = '\0';
 	remotehost[0] = '\0';
-	while ((c = getopt(argc, argv, "c:deh:l:p:rsxDET:VW78")) != EOF) {
+	while ((c = getopt(argc, argv, "c:deh:l:p:PrsxDET:VW78")) != EOF) {
 	  switch (c) {
 	  case 'c':		/* specify channel scanned for */
 	    channel = strdup(optarg);
@@ -399,6 +408,9 @@ main(argc, argv)
 	  case 'p':		/* server port */
 	    servport = atoi(optarg);
 	    break;
+	  case 'P':
+	    no_pipelining = 1;	/* It doesn't handle it.. */
+	    break;
 	  case 'r':		/* use reserved port for SMTP connection */
 	    wantreserved = 1;
 	    break;
@@ -485,6 +497,7 @@ main(argc, argv)
 	/* defer opening a connection until we know there is work */
 	smtpstatus = EX_DATAERR;
 	smtp_isopen = 0;
+	idle = 0;
 
 	while (!getout) {
 	  /* Input:
@@ -495,16 +508,23 @@ main(argc, argv)
 	  printf("#hungry\n");
 	  fflush(stdout);
 
-	  if (statusreport) report("#hungry");
+	  if (statusreport)
+	    if (idle)
+	      report("#idle");
+	    else
+	      report("#hungry");
 
 	  if (fgets(file, sizeof file, stdin) == NULL) break;
+	  idle = 0;
 	  if (strchr(file, '\n') == NULL) break; /* No ending '\n' !  Must
 						    have been partial input! */
 	  if (logfp) {
 	    fprintf(logfp,"%s#\tjobspec: %s",logtag(),file);
 	  }
-	  if (strcmp(file, "#idle\n") == 0)
+	  if (strcmp(file, "#idle\n") == 0) {
+	    idle = 1;
 	    continue; /* XX: We can't stay idle for very long, but.. */
+	  }
 	  if (emptyline(file, sizeof file))
 	    break;
 
@@ -709,7 +729,8 @@ deliver(dp, smtpfp, verboselog, startrp, endrp)
 	struct rcpt *startrp, *endrp;
 {
 	struct rcpt *rp = NULL;
-	int r, nrcpt, rcpt_cnt, hsize, size, tout;
+	int r = EX_TEMPFAIL;
+	int nrcpt, rcpt_cnt, hsize, size, tout;
 	int content_kind = 0, convertmode;
 	int ascii_clean = 0;
 	struct stat stbuf;
@@ -721,6 +742,8 @@ deliver(dp, smtpfp, verboselog, startrp, endrp)
 	time_t env_start, body_start, body_end;
 	struct rcpt *more_rp = NULL;
 
+	if (no_pipelining) pipelining = 0;
+
 	convertmode = _CONVERT_NONE;
 	if (conv_prohibit >= 0) {
 
@@ -791,7 +814,6 @@ deliver(dp, smtpfp, verboselog, startrp, endrp)
 	  more_rp = NULL;
 	}
 
-
 	if (strcmp(startrp->addr->link->channel,"error")==0)
 	  sprintf(SMTPbuf, "MAIL From:<>");
 	else
@@ -822,7 +844,7 @@ deliver(dp, smtpfp, verboselog, startrp, endrp)
 	r = smtpwrite(smtpfp, verboselog, 1, SMTPbuf,
 		      pipelining ? startrp : NULL);
 	if (r != EX_OK) {
-	  if (pipelining)
+	  if (pipelining && smtp_isopen /* might have timed out */)
 	    r = smtp_sync(smtpfp, verboselog, r);
 	  for (rp = startrp; rp && rp != endrp; rp = rp->next) {
 	    /* NOTARY: addres / action / status / diagnostic */
@@ -871,7 +893,7 @@ deliver(dp, smtpfp, verboselog, startrp, endrp)
  	  r = smtpwrite(smtpfp, verboselog, 1, SMTPbuf,
 			pipelining ? rp : NULL);
 	  if (r != EX_OK) {
-	    if (pipelining)
+	    if (pipelining && smtp_isopen)
 	      r = smtp_sync(smtpfp, verboselog, r);
 	    /* NOTARY: addres / action / status / diagnostic / wtt */
 	    notaryreport(NULL, notaryacct(r,"*NOTOK"),NULL,NULL);
@@ -890,6 +912,8 @@ deliver(dp, smtpfp, verboselog, startrp, endrp)
 	    r = smtpwrite(smtpfp, verboselog, 0, "RSET", NULL);
 
 	  if (r == EX_OK && more_rp)
+	    /* we have more recipients,
+	       and things have worked ok so far.. */
 	    goto more_recipients;
 
 	  return EX_UNAVAILABLE;
@@ -901,7 +925,8 @@ deliver(dp, smtpfp, verboselog, startrp, endrp)
 	  /* In PIPELINING mode ... send "DATA" */
 	  r = smtpwrite(smtpfp, verboselog, 1, "DATA", startrp);
 	  if (r != EX_OK) { /* failure on pipes ? */
-	    r = smtp_sync(smtpfp, verboselog, r); /* Sync it.. */
+	    if (smtp_isopen)
+	      r = smtp_sync(smtpfp, verboselog, r); /* Sync it.. */
 	    for (rp = startrp; rp && rp != endrp; rp = rp->next)
 	      if (rp->status == EX_OK) {
 		/* NOTARY: addres / action / status / diagnostic / wtt */
@@ -913,7 +938,8 @@ deliver(dp, smtpfp, verboselog, startrp, endrp)
 	    return r;
 	  }
 	  /* Now it is time to do synchronization .. */
-	  r = smtp_sync(smtpfp, verboselog, EX_OK); /* Up & until "DATA" .. */
+	  if (smtp_isopen)
+	    r = smtp_sync(smtpfp, verboselog, EX_OK); /* Up & until "DATA".. */
 	  if (r != EX_OK) {
 	    /* XX:
 	       #error  Uncertain of what to do ...
@@ -1980,7 +2006,7 @@ vcsetup(ipaddrcp, ipaddrlen, fdp, hostname, verboselog)
 	  int on = 1;
 /* setreuid(0,0); */
 	  *fdp = s;
-#if	BSD >= 43
+#if	defined(__svr4__) || defined(BSD) && (BSD-0) >= 43
 	  setsockopt(s, SOL_SOCKET, SO_KEEPALIVE, &on, sizeof on);
 #else  /* BSD < 43 */
 	  setsockopt(s, SOL_SOCKET, SO_KEEPALIVE, 0, 0);
@@ -2108,9 +2134,6 @@ smtp_flush()
 #include <sys/file.h>
 #endif
 #include <errno.h>
-#include <netdb.h>
-#include <netinet/in.h>
-#include <sys/socket.h>
 #include <sys/time.h>
 
 #ifndef	NFDBITS
@@ -2131,6 +2154,11 @@ typedef long	fd_mask;
 typedef	struct fd_set { fd_mask	fds_bits[1]; } fd_set;
 #endif	/* notdef */
 
+#ifdef _AIX
+#include <sys/types.h>
+#include <sys/select.h>
+#endif
+
 #ifndef	FD_SET
 #define	FD_SET(n, p)	((p)->fds_bits[0] |= (1 << (n)))
 #define	FD_CLR(n, p)	((p)->fds_bits[0] &= ~(1 << (n)))
@@ -2139,9 +2167,7 @@ typedef	struct fd_set { fd_mask	fds_bits[1]; } fd_set;
 #endif	/* !FD_SET */
 #endif	/* !NFDBITS */
 
-#ifdef _AIX
-#include <sys/types.h>
-#include <sys/select.h>
+#if defined(_AIX) && !defined(FD_SET) /* Pre AIX 4.1 ? */
 #define _FD_SET(sock,var) FD_SET(sock,&var.fdsmask)
 #define _FD_CLR(sock,var) FD_CLR(sock,&var.fdsmask)
 #define _FD_ZERO(var) FD_ZERO(&var.fdsmask)
@@ -2254,6 +2280,10 @@ char **statusp;
 		status = "5.1.3 (bad destination mailbox address syntax)";
 		rc = EX_NOUSER;
 		break;
+	case 554:
+		status = "5.1.1 (No acceptable recipients given)";
+		rc = EX_NOUSER;
+		break;
 	case 555: /* Unknown MAIL FROM:<>/RCPT TO:<> parameter */
 		status = "5.5.4 (invalid parameters)";
 		rc = EX_USAGE;
@@ -2278,6 +2308,7 @@ char **statusp;
 			rc = EX_TEMPFAIL;
 			break;
 		}
+		break;
 	}
 	*statusp = status;
 	return rc;
@@ -2303,8 +2334,9 @@ int smtp_sync(smtpfp,verboselog,r)
 	s = pipebuf;
 	eol = s;
 	for (idx = 0; idx < pipeindex; ++idx) {
+      rescan_line_0: /* processed some continuation line */
 	  s = eol;
-      rescan_line:
+      rescan_line:   /* Got additional input */
 	  eof = pipebuf + pipebufsize;
 	  for (eol = s; eol < eof; ++eol)
 	    if (*eol == '\n') break;
@@ -2338,10 +2370,10 @@ int smtp_sync(smtpfp,verboselog,r)
 	      /* XX: what to do with the error ? */
 	      if (logfp)
 		fprintf(logfp,"%s#\tRemote gave error %d (%s) while %d responces missing\n",
-			logtag(), err, strerror(err), pipeindex - idx);
+			logtag(), err, strerror(err), pipeindex - 1 - idx);
 	      if (verboselog)
 		fprintf(logfp,"Remote gave error %d (%s) while %d responces missing\n",
-			err, strerror(err), pipeindex - idx);
+			err, strerror(err), pipeindex - 1 - idx);
 	      break;
 	    } else if (len == 0) {
 	      /* The remote hung up! */
@@ -2383,12 +2415,12 @@ int smtp_sync(smtpfp,verboselog,r)
 	      s[1] >= '0' && s[1] <= '9' &&
 	      s[2] >= '0' && s[2] <= '9' &&
 	      s[3] == ' ') {
-	    code = atoi(s-3);
+	    code = atoi(s);
 
 	    /* We have a 'terminal' line */
 
 	  } else /* it is 'continuation line', or some such, ignore */
-	    goto rescan_line;
+	    goto rescan_line_0;
 
 	  if (pipecmds[idx]) {
 	    if (strncmp(pipecmds[idx],"RSET",4) != 0) /* If RSET,
@@ -2408,7 +2440,7 @@ int smtp_sync(smtpfp,verboselog,r)
 	    if (idx == 0) {
 	      /* MAIL FROM:<*>: ... */
 	      rc = code_to_status(code, &status);
-	    } else if (idx == pipeindex-1) {
+	    } else if (idx == pipeindex-2) {
 	      /* DATA: 354/ 451/554/ 500/501/503/421 */
 	      rc = code_to_status(code, &status);
 	    } else {
@@ -2416,14 +2448,17 @@ int smtp_sync(smtpfp,verboselog,r)
 	      rc = code_to_status(code, &status);
 	    }
 	    /* Diagnose the errors, we report successes AFTER the DATA phase.. */
+	    notarystatsave(s,NULL);
 	    notaryreport(pipercpts[idx]->addr->user,FAILED,
-			 status,remotemsg);
+			 status,NULL);
 	    diagnostic(pipercpts[idx],rc, 0, "%s", remotemsg);
 	  } else {
 	    /* Ok results */
 	    pipercpts[idx]->status = EX_OK;
-	    if (idx != 0 && idx != pipeindex-1)
+	    if (idx != 0 && idx != pipeindex-2) {
 	      some_ok = 1;
+if (verboselog) fprintf(verboselog,"[Some OK - code=%d, idx=%d, pipeindex=%d]\n",code,idx,pipeindex-1);
+	    }
 	  }
 	  if (pipecmds[idx] != NULL)
 	    free(pipecmds[idx]);
@@ -2646,22 +2681,6 @@ smtpwrite(smtpfp, verboselog, saverpt, strbuf, syncrp)
 		i = (*s == '-');
 		break;
 	      }
-	      if (debug) {
-		if (i%8 == 0) {
-		  if (i > 0)
-		    putc('\n',logfp);
-		  fprintf(logfp,
-			  "%s#\t", logtag());
-		}
-		if (isprint(*s)) 
-		  putc(*s, logfp);
-		else
-		  fprintf(logfp,
-			  "\\%03o", *s);
-	      }
-	    }
-	    if (debug) {
-	      fprintf(logfp, "\n");
 	    }
 	  } else if (r == -1) {
 	    if (gotalarm) {
@@ -3067,6 +3086,7 @@ getmxrr(host, mx, maxmx, depth, mxcntp)
 	  if (i == nmx) {	/* we are the best MX, do it another way */
 	    mx[0].host = NULL;
 	    mxcount = 0;
+	    *mxcntp = 0;
 	    return EX_OK;
 	  }
 	}
@@ -3144,6 +3164,7 @@ getmxrr(host, mx, maxmx, depth, mxcntp)
 #endif
 	mx[nmx].host = NULL;
 	mxcount = nmx;
+	*mxcntp = nmx;
 	return EX_OK;
 }
 
@@ -3295,16 +3316,10 @@ char *smtpline, *status;
 	*s++ = *smtpline++;
 	*s++ = *smtpline++;
 	*s++ = ' ';
-	if (len < 11) {
-	  notaryreport(NULL,NULL,status,str);
-#ifndef USE_ALLOCA
-	  free(str);
-#endif
-	  return;
-	}
-	*s++ = '(';
-	while (*smtpline) {
-	  switch (*smtpline) {
+	if (len >= 11) {
+	  *s++ = '(';
+	  while (*smtpline) {
+	    switch (*smtpline) {
 	    case '(':
 	        *s++ = '[';
 		break;
@@ -3313,10 +3328,11 @@ char *smtpline, *status;
 		break;
 	    default:
 		*s++ = *smtpline;
+	    }
+	    ++smtpline;
 	  }
-	  ++smtpline;
+	  *s++ = ')';
 	}
-	*s++ = ')';
 	*s = 0;
 	notaryreport(NULL,NULL,status,str);
 #ifndef USE_ALLOCA
